# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import pytest

from radish.stepmodel import Step
from radish.exceptions import RadishError


def test_creating_simple_step():
    """
    Test creating a simple Step
    """
    # given & when
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=None, runable=True, context_class=None)

    # then
    assert step.id == 1
    assert step.sentence == 'I am a Step'
    assert step.path == 'foo.feature'
    assert step.line == 1
    assert step.parent is None
    assert step.runable is True
    assert step.context_class is None


def test_getting_step_context_object(mocker):
    """
    Test getting a Step's context object
    """
    # given
    scenario = mocker.MagicMock(context=42)
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=scenario, runable=True, context_class=None)

    # when
    context = step.context

    # then
    assert context == 42
    assert context == scenario.context


def test_getting_expanded_sentence(mocker):
    """
    Test getting the expanded Step sentence
    """
    # given
    scenario = mocker.MagicMock(constants=[
        ('foo', '42'), ('bar', '21'),
        ('bla', '33')
    ])
    step = Step(1, 'I am ${foo} and ${bar} bla', 'foo.feature', 1, parent=scenario, runable=True, context_class=None)

    # when
    sentence = step.expanded_sentence

    # then
    assert sentence == 'I am 42 and 21 bla'
    assert step.sentence == 'I am ${foo} and ${bar} bla'


def test_getting_context_sensitive_sentence(mocker):
    """
    Test getting the context sensitive Step sentence
    """
    # given
    scenario = mocker.MagicMock(constants=[
        ('foo', '42'), ('bar', '21'),
        ('bla', '33')
    ])
    step = Step(1, 'And I am ${foo} and ${bar} bla', 'foo.feature', 1, parent=scenario, runable=True, context_class='Given')
    step_no_context = Step(1, 'And I am ${foo} and ${bar} bla', 'foo.feature', 1, parent=scenario, runable=True, context_class=None)

    # when
    sentence = step.context_sensitive_sentence
    sentence_no_context = step_no_context.context_sensitive_sentence

    # then
    assert sentence == 'Given I am 42 and 21 bla'
    assert step.sentence == 'And I am ${foo} and ${bar} bla'

    assert sentence_no_context == 'And I am 42 and 21 bla'
    assert step_no_context.sentence == 'And I am ${foo} and ${bar} bla'


@pytest.mark.parametrize('given_raw_text, expected_text', [
    ([], ''),
    (['Hello', 'World'], 'Hello\nWorld'),
    (['Hello', 'Awesome', 'World'], 'Hello\nAwesome\nWorld')
])
def test_getting_raw_text_from_step(given_raw_text, expected_text):
    """
    Test getting a Step's raw text data
    """
    # given
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=None, runable=True, context_class=None)
    step.raw_text = given_raw_text

    # when
    text = step.text

    # then
    assert text == expected_text


@pytest.mark.parametrize('debug_or_run', [
    ('debug'), ('run')
])
def test_run_not_runable_step(debug_or_run, mock_utils_debugger):
    """
    Test running a non-runable Step
    """
    # given
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=None, runable=False, context_class=None)

    # when
    method = getattr(step, debug_or_run)
    state = method()

    # then
    assert state == Step.State.UNTESTED


def test_run_step_with_invalid_defintion_func():
    """
    Test running a Step with an invalid definition function
    """
    # given
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=None, runable=True, context_class=None)

    # when the Step doesn't have a definition function
    with pytest.raises(RadishError) as exc:
        step.run()
    # then the Step fails with an Exception
    assert str(exc.value) == "The step 'I am a Step' does not have a step definition"

    # when the Step has a non-callable definition function
    step.definition_func = 'not-callable'
    with pytest.raises(RadishError) as exc:
        step.run()
    # then the step fails with an Exception
    assert str(exc.value) == "The step 'I am a Step' does not have a step definition"


class StepHelper(object):
    """Helper class only used for mocking"""
    @staticmethod
    def step_func(step, foo=None, bar=None):
        """
        Helper Step Definition Function.
        """
        pass

    @staticmethod
    def step_pending_func(step):
        """
        Helper Step Definition Function which
        sets the Step to PENDING
        """
        step.pending()

    @staticmethod
    def step_fail_func(step):
        """
        Helper Step Definition Function which fails the Step
        """
        raise AssertionError('failing step')


@pytest.mark.parametrize('debug_or_run', [
    ('run'), ('debug')
])
def test_run_debug_step_function_with_kwargs(debug_or_run, mocker, mock_utils_debugger):
    """
    Test running/debugging a Step with a function and keyword arguments
    """
    # mock step function which is to use
    mocker.spy(StepHelper, 'step_func')

    # given
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=None, runable=True, context_class=None)
    step.definition_func = StepHelper.step_func
    step.argument_match = mocker.MagicMock()
    step.argument_match.evaluate.return_value = (tuple(), {'foo': '1', 'bar': '2'})

    # when
    method = getattr(step, debug_or_run)
    state = method()

    # then
    assert state == Step.State.PASSED
    StepHelper.step_func.assert_called_once_with(step, foo='1', bar='2')


@pytest.mark.parametrize('debug_or_run', [
    ('run'), ('debug')
])
def test_run_debug_step_function_with_posargs(debug_or_run, mocker, mock_utils_debugger):
    """
    Test running/debugging a Step with a function and positional arguments
    """
    # mock step function which is to use
    mocker.spy(StepHelper, 'step_func')

    # given
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=None, runable=True, context_class=None)
    step.definition_func = StepHelper.step_func
    step.argument_match = mocker.MagicMock()
    step.argument_match.evaluate.return_value = ((1, 2), {})

    # when
    method = getattr(step, debug_or_run)
    state = method()

    # then
    assert state == Step.State.PASSED
    StepHelper.step_func.assert_called_once_with(step, 1, 2)


@pytest.mark.parametrize('debug_or_run', [
    ('run'), ('debug')
])
def test_run_debug_step_function_mark_pending(debug_or_run, mocker, mock_utils_debugger):
    """
    Test running/debugging a Step which marks itself as pending
    """
    # given
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=None, runable=True, context_class=None)
    step.definition_func = StepHelper.step_pending_func
    step.argument_match = mocker.MagicMock()
    step.argument_match.evaluate.return_value = (tuple(), {})

    # when
    method = getattr(step, debug_or_run)
    state = method()

    # then
    assert state == Step.State.PENDING == step.state


@pytest.mark.parametrize('debug_or_run', [
    ('run'), ('debug')
])
def test_run_debug_step_function_with_exception(debug_or_run, mocker, mock_utils_debugger):
    """
    Test running/debugging a Step which raises an Exception
    """
    # given
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=None, runable=True, context_class=None)
    step.definition_func = StepHelper.step_fail_func
    step.argument_match = mocker.MagicMock()
    step.argument_match.evaluate.return_value = (tuple(), {})

    # when
    method = getattr(step, debug_or_run)
    state = method()

    # then
    assert state == step.state == Step.State.FAILED
    assert step.failure is not None
    assert step.failure.reason == 'failing step'
    assert step.failure.name == 'AssertionError'


def test_skip_a_step():
    """
    Test skipping a Step
    """
    # given
    step = Step(1, 'I am a Step', 'foo.feature', 1, parent=None, runable=True, context_class=None)

    # when
    step.skip()

    # then
    assert step.state == Step.State.SKIPPED



# -*- coding: utf-8 -*-

"""
This module provides a script to test the
radish step implementations
"""

import sys

import colorful
from docopt import docopt

from radish import __VERSION__
import radish.utils as utils
from radish.testing.matches import test_step_matches_configs


# use only 8 ANSI colors
# FIXME(TF): change to true colors!
colorful.use_8_ansi_colors()


def main():
    """
Usage:
    radish-test matches <match-configs>...
        [-b=<basedir> | --basedir=<basedir>...]
        [--cover-min-percentage=<cover-min-percentage>]
        [--cover-show-missing]
    radish-test (-h | --help)
    radish-test (-v | --version)

Arguments:
    match-configs                                  configuration files of step matcher tests

Commands:
    matches                                        test if the step implementations actually match the expected sentences

Options:
    -h --help                                      show this screen
    -v --version                                   show version
    -b=<basedir> --basedir=<basedir>...            set base dir from where the step.py and terrain.py will be loaded. [default: $PWD/radish]
                                                   You can specify -b|--basedir multiple times or split multiple paths with a colon (:) similar to $PATH. All files will be imported.
    --cover-min-percentage=<cover-min-percentage>  minimum percentage of step coverage for tests to pass
    --cover-show-missing                           show steps which are not tested

(C) Copyright by Timo Furrer <tuxtimo@gmail.com>
    """

    arguments = docopt("radish-test {0}\n{1}".format(__VERSION__, main.__doc__), version=__VERSION__)


    if arguments['matches']:
        return test_step_matches_configs(arguments['<match-configs>'],
             utils.flattened_basedirs(arguments['--basedir']), arguments['--cover-min-percentage'], arguments['--cover-show-missing'])


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-

from .terrain import world


__DOCS__ = "https://github.com/radish-bdd/radish"


class RadishError(Exception):
    """
        General radish specific error
    """
    pass


class LanguageNotSupportedError(RadishError):
    """
        Raised if the language could not be found.
    """
    def __init__(self, language):
        self.language = language
        super(LanguageNotSupportedError, self).__init__("Language {0} could not be found".format(language))


class FeatureFileNotFoundError(RadishError):
    """
        Raised if a given feature file does not exist
    """
    def __init__(self, featurefile):
        self.featurefile = featurefile
        super(FeatureFileNotFoundError, self).__init__("Feature file '{0}': No such file".format(featurefile))


class FeatureFileSyntaxError(RadishError, SyntaxError):
    """
        If a a syntax error occured in a feature file
    """
    MESSAGE_TEMPLATE = """{msg}

Error Oracle says:
You have a SyntaxError in your feature file!
Please have a look into the radish documentation to found out which
features radish supports and how you could use them:
Link: {docs_link}"""

    def __init__(self, msg):
        super(FeatureFileSyntaxError, self).__init__(FeatureFileSyntaxError.MESSAGE_TEMPLATE.format(msg=msg, docs_link=__DOCS__))


class StepRegexError(RadishError, SyntaxError):
    """
        Raised if the step regex cannot be compiled.
    """
    def __init__(self, regex, step_func_name, re_error):
        self.regex = regex
        self.step_func_name = step_func_name
        self.re_error = re_error
        super(StepRegexError, self).__init__("Cannot compile regex '{0}' from step '{1}': {2}".format(regex, step_func_name, re_error))


class StepPatternError(RadishError, SyntaxError):
    """
        Raised if the steps pattern cannot be compiled.
    """
    def __init__(self, pattern, step_func_name, error):
        self.pattern = pattern
        self.step_func_name = step_func_name
        self.error = error
        super(StepPatternError, self).__init__("Cannot compile pattern '{0}' of step '{1}': {2}".format(pattern, step_func_name, error))


class SameStepError(RadishError):
    """
        Raised if two step regex are exactly the same.
    """
    MESSAGE_TEMPLATE = """Cannot register step {0} with regex '{1}' because it is already used by step {2}

Error Oracle says:
You have defined two step definitions with the same Regular Expression.
This is invalid since radish does not know which one is the one to go with.
If you have two similar step definition expressions but ones sentence is a subset of the other
you may want to add a $ to mark the sentence's end - take care of the code order - first comes first serves."""

    def __init__(self, regex, func1, func2):
        self.regex = regex
        self.func1 = func1
        self.func2 = func2
        super(SameStepError, self).__init__(SameStepError.MESSAGE_TEMPLATE.format(func2.__name__, regex, func1.__name__))


class StepDefinitionNotFoundError(RadishError):
    """
        Raised if the Matcher cannot find a step from the feature file in the registered steps.
    """
    MESSAGE_TEMPLATE = """Cannot find step definition for step '{sentence}' in {step_path}:{step_line}

Error Oracle says:
There is no step defintion for '{sentence}'.
All steps should be declared in a module located in {basedir}.
For example you could do:

@step(r"{sentence}")
def my_step(step):
    raise NotImplementedError("This step is not implemented yet")"""

    def __init__(self, step):
        self.step = step
        super(StepDefinitionNotFoundError, self).__init__(StepDefinitionNotFoundError.MESSAGE_TEMPLATE.format(
            sentence=step.sentence, step_path=step.path, step_line=step.line, basedir=' or '.join(world.config.basedir)))


class RunnerEarlyExit(RadishError):
    """
        Raised if the runner has to exit to run.
    """
    pass


class HookError(RadishError):
    """
        Raised if an exception was raised inside a hook
    """
    def __init__(self, hook_function, failure):
        self.hook_function = hook_function
        self.failure = failure
        super(HookError, self).__init__("Hook '{0}' from {1}:{2} raised: '{3}: {4}'".format(
            hook_function.__name__, hook_function.__code__.co_filename, hook_function.__code__.co_firstlineno, failure.name, failure.reason))


class ScenarioNotFoundError(RadishError):
    """
        Raised if a scenario cannot be found
    """
    def __init__(self, scenario_id, amount_of_scenarios):
        self.scenario_id = scenario_id
        self.amount_of_scenarios = amount_of_scenarios
        super(ScenarioNotFoundError, self).__init__("No scenario with id {0} found. Specify a scenario id between 1 and {1}".format(scenario_id, amount_of_scenarios))


class ValidationError(RadishError):
    """
        Raised by the user if a step is somehow not valid
    """
    pass



# -*- coding: utf-8 -*-

"""
    This module provides a registry for all hooks
"""

from singleton import singleton

from . import utils
from .exceptions import HookError

import tagexpressions


@singleton()
class HookRegistry(object):
    """
        Represents an object with all registered hooks
    """
    def __init__(self):
        self._hooks = {}
        self.reset()
        self.build_hooks()

    @property
    def hooks(self):
        """
            Returns all registered hooks
        """
        return self._hooks

    class Hook(object):
        """
            Represents a hook object

            This object is needed to provide decorators like:
            * @before.all
            * @before.each_feature
        """
        def __init__(self, when):
            self._when = when

        @classmethod
        def build_decorator(cls, what):
            """
                Builds the hook decorator
            """
            def _decorator(self, *args, **kwargs):
                """
                    Actual hook decorator
                """
                if len(args) == 1 and len(kwargs) == 0 and callable(args[0]):
                    func = args[0]
                    # hook was called without argument -> legacy!
                    HookRegistry().register(self._when, what, func)  # pylint: disable=protected-access
                else:
                    # hook was called with argument
                    on_tags = kwargs.get('on_tags')

                    if on_tags:
                        expr = tagexpressions.parse(on_tags)
                        on_tags = lambda tags: expr.evaluate(tags)

                    def func(f):
                        HookRegistry().register(self._when, what, f, on_tags)
                        return f

                return func
            _decorator.__name__ = _decorator.fn_name = what
            setattr(cls, what, _decorator)

    def build_hooks(self):
        """
            Builds all hooks
        """
        for hook in self._hooks.keys():
            self.Hook.build_decorator(hook)

    def register(self, when, what, func, on_tags=None):
        """
            Registers a function as a hook
        """
        if on_tags is None:
            on_tags = lambda _: True  # if no tags are specified we always return True

        self._hooks[what][when].append((on_tags, func))

    def reset(self):
        """
            Resets all registerd hooks
        """
        self._hooks = {
            "all": {"before": [], "after": []},
            "each_feature": {"before": [], "after": []},
            "each_scenario": {"before": [], "after": []},
            "each_step": {"before": [], "after": []},
        }

    def __has_to_run(self, model, on_tags):
        """
        Return if the given hook has to run or not
        depending on it's tags
        """
        if isinstance(model, list):
            return any(on_tags([t.name for t in m.all_tags]) for m in model)

        return on_tags([t.name for t in model.all_tags])


    def call(self, when, what, model, *args, **kwargs):
        """
            Calls a registered hook
        """
        for on_tags, func in self._hooks[what][when]:
            if not self.__has_to_run(model, on_tags):
                # # this hook does not have to run because
                # # it was excluded due to the tags for this model
                continue

            try:
                func(model, *args, **kwargs)
            except Exception as e:
                raise HookError(func, utils.Failure(e))
        return None

HookRegistry()
before = HookRegistry.Hook("before")  # pylint: disable=invalid-name
after = HookRegistry.Hook("after")  # pylint: disable=invalid-name



# -*- coding: utf-8 -*-

"""
    This module is a REQUIRED extension to record the time of Features, Scenarios and Steps
"""

from datetime import datetime

from radish.hookregistry import after, before
from radish.extensionregistry import extension

__REQUIRED__ = True

@extension
class TimeRecorder(object):
    """
        Time Recorder radish plugin
    """
    LOAD_IF = staticmethod(lambda config: not config.show)
    LOAD_PRIORITY = 1

    def __init__(self):
        before.each_feature(self.time_recorder_before_each_feature)
        before.each_scenario(self.time_recorder_before_each_scenario)
        before.each_step(self.time_recorder_before_each_step)
        after.each_feature(self.time_recorder_after_each_feature)
        after.each_scenario(self.time_recorder_after_each_scenario)
        after.each_step(self.time_recorder_after_each_step)

    def time_recorder_before_each_feature(self, feature):
        """
            Sets the starttime of the feature
        """
        feature.starttime = datetime.now()

    def time_recorder_before_each_scenario(self, scenario):
        """
            Sets the starttime of the scenario
        """
        scenario.starttime = datetime.now()


    def time_recorder_before_each_step(self, step):
        """
            Sets the starttime of the step
        """
        step.starttime = datetime.now()

    def time_recorder_after_each_feature(self, feature):
        """
            Sets the endtime of the feature
        """
        feature.endtime = datetime.now()

    def time_recorder_after_each_scenario(self, scenario):
        """
            Sets the endtime of the scenario
        """
        scenario.endtime = datetime.now()

    def time_recorder_after_each_step(self, step):
        """
            Sets the endtime of the step
        """
        step.endtime = datetime.now()



# -*- coding: utf-8 -*-

from radish import custom_type


@custom_type('is_plugged_in', r'(plugged in|not plugged in)')
def is_plugged_in(text):
    return text == 'plugged in'


@custom_type('is_registered', r'(not )?to be registered')
def is_registered(text):
    return text == 'to be registered'



# -*- coding: utf-8 -*-

"""
    This module provides a Scenario type which represents a Scenario loop
"""

import copy

from .scenario import Scenario
from .iterationscenario import IterationScenario
from .stepmodel import Step


class ScenarioLoop(Scenario):
    """
        Represents a scenario loop
    """
    def __init__(self, id, keyword, iterations_keyword, sentence, path, line, parent, tags=None, preconditions=None, background=None):
        super(ScenarioLoop, self).__init__(
                id, keyword, sentence, path, line, parent,
                tags, preconditions, background=background)
        self.iterations_keyword = iterations_keyword
        self.iterations = 0
        self.scenarios = []

    def build_scenarios(self):
        """
            Builds the scenarios for every iteration

            Note: This must be done before mering the steps
                  from the feature file with the step definitions
        """
        for i in range(self.iterations):
            scenario_id = self.id + i + 1
            background = None
            scenario = IterationScenario(
                    scenario_id, self.keyword,
                    "{0} - iteration {1}".format(self.sentence, i), self.path, self.line, self, i)
            if self.background:
                background = self.background.create_instance(parent=scenario, steps_runable=True)
                scenario.background = background

            for step_id, iteration_step in enumerate(self.steps):
                step = Step(
                        step_id + 1, iteration_step.sentence, iteration_step.path,
                        iteration_step.line, scenario, True,
                        context_class=iteration_step.context_class)
                step.table_header = copy.copy(iteration_step.table_header)
                step.table_data = copy.copy(iteration_step.table_data)
                step.table = copy.copy(iteration_step.table)
                step.raw_text = copy.copy(iteration_step.raw_text)
                scenario.steps.append(step)
            self.scenarios.append(scenario)

    def after_parse(self):
        """
            Build looped scenarios
        """
        Scenario.after_parse(self)
        self.build_scenarios()
        self.complete = True



# -*- coding: utf-8 -*-

"""
    This module provides a hook which generates a BDD XML result file at the end of the run.
"""

from getpass import getuser
from socket import gethostname
from datetime import timedelta
import re

from radish.terrain import world
from radish.hookregistry import after
from radish.exceptions import RadishError
from radish.scenariooutline import ScenarioOutline
from radish.scenarioloop import ScenarioLoop
from radish.stepmodel import Step
from radish.extensionregistry import extension
from radish.exceptions import RadishError
import radish.utils as utils


@extension
class BDDXMLWriter(object):
    """
        BDD XML Writer radish extension
    """
    OPTIONS = [("--bdd-xml=<bddxml>", "write BDD XML result file after run")]
    LOAD_IF = staticmethod(lambda config: config.bdd_xml)
    LOAD_PRIORITY = 60

    def __init__(self):
        try:
            from lxml import etree
        except ImportError:
            raise RadishError('if you want to use the BDD xml writer you have to "pip install radish-bdd[bdd-xml]"')

        after.all(self.generate_bdd_xml)

    def _get_element_from_model(self, what, model):
        """
            Create a etree.Element from a given model
        """
        from lxml import etree

        # round duration to 10 decimal points, to avoid it being printed in
        # scientific notation
        duration = "%.10f" % model.duration.total_seconds() if model.starttime and model.endtime else ""
        return etree.Element(
            what,
            sentence=model.sentence,
            id=str(model.id),
            result=model.state,
            starttime=utils.datetime_to_str(model.starttime),
            endtime=utils.datetime_to_str(model.endtime),
            duration=duration,
            testfile=model.path
        )

    def _strip_ansi(self, text):
        """
            Strips ANSI modifiers from the given text
        """
        pattern = re.compile(r"(\\033\[\d+(?:;\d+)*m)")
        return pattern.sub("", text)

    def generate_bdd_xml(self, features, marker):
        """
            Generates the bdd xml
        """
        from lxml import etree
        if not features:
            raise RadishError("No features given to generate BDD xml file")

        duration = timedelta()
        for feature in features:
            if feature.state in [Step.State.PASSED, Step.State.FAILED]:
                duration += feature.duration

        testrun_element = etree.Element(
            "testrun",
            starttime=utils.datetime_to_str(features[0].starttime),
            endtime=utils.datetime_to_str(features[-1].endtime),
            duration=str(duration.total_seconds()),
            agent="{0}@{1}".format(getuser(), gethostname())
        )

        for feature in features:
            if not feature.has_to_run(world.config.scenarios):
                continue

            feature_element = self._get_element_from_model("feature", feature)

            description_element = etree.Element("description")
            description_element.text = etree.CDATA("\n".join(feature.description))

            tags_element = etree.Element('tags')

            for tag in feature.tags:
                tag_element = etree.Element('tag')
                tag_element.text = tag.name
                tags_element.append(tag_element)

            scenarios_element = etree.Element("scenarios")

            for scenario in (s for s in feature.all_scenarios if not isinstance(s, (ScenarioOutline, ScenarioLoop))):
                if not scenario.has_to_run(world.config.scenarios):
                    continue
                scenario_element = self._get_element_from_model("scenario", scenario)

                scenario_tags_element = etree.Element('tags')
                scenario_element.append(scenario_tags_element)

                for tag in scenario.tags:
                    tag_element = etree.Element('tag')
                    tag_element.text = tag.name
                    if tag.arg:
                        tag_element.text += '({0})'.format(tag.arg)
                    scenario_tags_element.append(tag_element)

                for step in scenario.all_steps:
                    step_element = self._get_element_from_model("step", step)
                    if step.state is Step.State.FAILED:
                        failure_element = etree.Element(
                            "failure",
                            type=step.failure.name,
                            message=step.failure.reason
                        )
                        failure_element.text = etree.CDATA(self._strip_ansi(step.failure.traceback))
                        step_element.append(failure_element)
                    scenario_element.append(step_element)
                scenarios_element.append(scenario_element)
            feature_element.append(description_element)
            feature_element.append(tags_element)
            feature_element.append(scenarios_element)
            testrun_element.append(feature_element)

        with open(world.config.bdd_xml, "w+") as f:
            content = etree.tostring(testrun_element, pretty_print=True, xml_declaration=True, encoding="utf-8")
            try:
                if not isinstance(content, str):
                    content = content.decode("utf-8")
            except Exception:
                pass
            finally:
                f.write(content)



# -*- coding: utf-8 -*-

"""
    This module provides a class to represent a Scenario
"""

from .model import Model
from .stepmodel import Step


class Scenario(Model):
    """
        Represents a Scenario
    """
    def __init__(self, id, keyword, sentence, path, line, parent, tags=None, preconditions=None, background=None):
        super(Scenario, self).__init__(id, keyword, sentence, path, line, parent, tags)
        self.absolute_id = None
        self.preconditions = preconditions or []
        self.background = background
        self.steps = []
        self.context = self.Context()
        self.complete = False

    @property
    def state(self):
        """
            Returns the state of the scenario
        """
        steps = []
        if self.background:
            steps.extend(self.background.steps)

        steps.extend(self.steps)
        for step in steps:
            if step.state is not Step.State.PASSED:
                return step.state
        return Step.State.PASSED

    @property
    def constants(self):
        """
            Returns all constants
        """
        constants = []
        for name, value in self.context.constants:
            for parent_name, parent_value in self.parent.constants:
                value = value.replace("${{{0}}}".format(parent_name), parent_value)
            constants.append((name, value))
        constants.extend(self.parent.constants)
        return constants

    @property
    def all_steps(self):
        """
            Returns all steps from all preconditions in the correct order
        """
        steps = []
        if self.background:
            steps.extend(self.background.all_steps)

        for precondition in self.preconditions:
            steps.extend(precondition.all_steps)
        steps.extend(self.steps)
        return steps

    @property
    def failed_step(self):
        """
            Returns the first failed step
        """
        steps = []
        if self.background:
            steps.extend(self.background.steps)

        steps.extend(self.steps)

        # FIXME(TF): what about Scenario Precondition Steps?

        for step in steps:
            if step.state == Step.State.FAILED:
                return step
        return None

    def has_to_run(self, scenario_choice):
        """
            Returns wheiter the scenario has to run or not

            :param list scenario_choice: the scenarios to run. If None all will run
        """
        if not scenario_choice:
            return True

        return self.absolute_id in (scenario_choice or [])

    def after_parse(self):
        """
            This method is called after the scenario is completely parsed.

            Actions to do:
              * number steps
              * fix parent of precondition steps
        """
        for step_id, step in enumerate(self.all_steps, start=1):
            step.id = step_id
            if step.parent != self:
                # check if step is from background
                if self.background and step in self.background.steps:
                    step.as_background = self.background
                else:
                    step.as_precondition = step.parent
                step.parent = self
        self.complete = True



# -*- coding: utf-8 -*-

from radish import given, when, then


@given('the device is {is_plugged_in:is_plugged_in}')
def device_plugged_in(step, is_plugged_in):
    step.context.powered = False
    step.context.device_plugged_in = is_plugged_in


@when('I turn on the board')
def turn_on_board(step):
    step.context.powered = True


@then('I expect the device {is_registered:is_registered}')
def device_registered(step, is_registered):
    assert step.context.powered, 'Board is not powered, thus device cannot be registered'
    # device is always registered when the device is plugged in
    assert step.context.device_plugged_in == is_registered, 'Device is not regsitered'



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import pytest

from radish.model import Tag
from radish.hookregistry import before, after
import radish.exceptions as errors


def test_available_hooks():
    """
    Test that all expected hooks are available
    """
    # then
    assert callable(before.all)
    assert callable(after.all)

    assert callable(before.each_feature)
    assert callable(after.each_feature)

    assert callable(before.each_scenario)
    assert callable(after.each_scenario)

    assert callable(before.each_step)
    assert callable(after.each_step)


def test_registering_simple_hook(hookregistry):
    """
    Test registering simple hooks
    """
    # given & when
    @before.all
    def before_all_hook_legacy():
        pass


    @before.all()
    def before_all_hook():
        pass

    # then
    assert len(hookregistry.hooks['all']['before']) == 2


def test_call_hook(hookregistry, mocker):
    """
    Test calling registered hooks
    """
    # given
    @before.all()
    def before_all(features, stub):
        stub()


    @before.each_step()
    def before_each_step(step, stub):
        stub()

    hook_call_stub = mocker.stub()

    # when & then
    hookregistry.call('before', 'all', mocker.MagicMock(), hook_call_stub)
    assert hook_call_stub.call_count == 1

    hookregistry.call('before', 'each_step', mocker.MagicMock(), hook_call_stub)
    assert hook_call_stub.call_count == 2


def test_call_hook_exception(hookregistry, mocker):
    """
    Test calling registered hook which raises an exception
    """
    # given
    @before.all()
    def before_all(features):
        raise AssertionError('some error')

    # when
    with pytest.raises(errors.HookError) as exc:
        hookregistry.call('before', 'all', mocker.MagicMock())

    # then
    assert exc.match(r'.*AssertionError: some error')


def test_call_hooks_filtered_by_tags(hookregistry, mocker):
    """
    Test calling filtered hooks by tags
    """
    # given
    @after.all()
    def generic_cleanup(features, stub):
        stub()

    @after.all(on_tags='bad_case')
    def bad_case_cleanup(features, stub):
        stub()

    @after.all(on_tags='good_case')
    def good_case_cleanup(features, stub):
        stub()

    hook_call_stub = mocker.stub()
    model = mocker.MagicMock(all_tags=[])

    models = [
        mocker.MagicMock(all_tags=[Tag(name='good_case')]),
        mocker.MagicMock(all_tags=[Tag(name='bad_case')])
    ]

    # when & then
    # all tags
    hookregistry.call('after', 'all', model, hook_call_stub)
    assert hook_call_stub.call_count == 1

    # only generic & good case
    model.all_tags = [Tag(name='good_case')]
    hookregistry.call('after', 'all', model, hook_call_stub)
    assert hook_call_stub.call_count == 3

    # only generic & bad case
    model.all_tags = [Tag(name='bad_case')]
    hookregistry.call('after', 'all', model, hook_call_stub)
    assert hook_call_stub.call_count == 5

    # good case & bad case because of model list
    hookregistry.call('after', 'all', models, hook_call_stub)
    assert hook_call_stub.call_count == 8



# -*- coding: utf-8 -*-

from radish import given, when, then


@given('I have the numbers {numbers:Number+}')
def have_number(step, numbers):
    step.context.numbers.extend(numbers)


@given('I have the following numbers {numbers:NumberList}')
def have_number(step, numbers):
    step.context.numbers.extend(numbers)


@when('I sum them')
def sum_numbers(step):
    step.context.result = sum(step.context.numbers)


@then('I expect the result to be {result:g}')
def expect_result(step, result):
    assert step.context.result == result


@given('I have studied the questions')
def studied_questions(step):
    pass


@when('I have to answer it')
def answer_question(step):
    pass


@then('I say "{answer:AnswerChoice}"')
def say_answer(step, answer):
    assert answer == 'yes'



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import pytest

from radish.extensionregistry import extension
import radish.exceptions as errors


def test_register_simple_extension_class(extensionregistry):
    """
    Test registering simple Extension class
    """
    # given
    class SimpleExtension(object):
        pass

    # when
    extensionregistry.register(SimpleExtension)

    # then
    assert len(extensionregistry.extensions) == 1
    assert extensionregistry.extensions[0] == SimpleExtension
    assert len(extensionregistry.loaded_extensions) == 0


def test_register_simple_extension_class_using_decorator(extensionregistry):
    """
    Test registering simple Extension class using the extension decorator
    """
    # given & when
    @extension
    class SimpleExtension(object):
        pass

    # then
    assert len(extensionregistry.extensions) == 1
    assert extensionregistry.extensions[0] == SimpleExtension
    assert len(extensionregistry.loaded_extensions) == 0


def test_loading_simple_extension(extensionregistry, mocker):
    """
    Test loading simple extension
    """
    # given
    @extension
    class SimpleExtension(object):
        LOAD_IF = staticmethod(lambda config: True)

    # when
    extensionregistry.load(mocker.MagicMock())

    # then
    assert len(extensionregistry.extensions) == 1
    assert extensionregistry.extensions[0] == SimpleExtension
    assert len(extensionregistry.loaded_extensions) == 1
    assert isinstance(extensionregistry.loaded_extensions[0], SimpleExtension)


# FIXME(TF): wrong behavior?!
def test_loading_invalid_extension(extensionregistry, mocker):
    """
    Test loading an invalid extension
    """
    # given
    @extension
    class SimpleExtension(object):
        pass

    # when
    extensionregistry.load(mocker.MagicMock())

    # then
    assert len(extensionregistry.extensions) == 1
    assert len(extensionregistry.loaded_extensions) == 0


def test_loading_extension_which_raises_exceptions_init(extensionregistry, mocker):
    """
    Test loading extension which raises exceptions in init
    """
    # given
    @extension
    class SimpleExtension(object):
        LOAD_IF = staticmethod(lambda config: True)
        def __init__(self):
            raise AssertionError('some error')

    # when
    with pytest.raises(AssertionError) as exc:
        extensionregistry.load(mocker.MagicMock())

    # then
    assert str(exc.value) == 'some error'


def test_loading_simple_extension_if_wanted(extensionregistry, mocker):
    """
    Test loading extension if wanted by config
    """
    # given
    @extension
    class WantedExtension(object):
        LOAD_IF = staticmethod(lambda config: True)

    @extension
    class UnwantedExtension(object):
        LOAD_IF = staticmethod(lambda config: False)

    # when
    extensionregistry.load(mocker.MagicMock())

    # then
    assert len(extensionregistry.extensions) == 2
    assert len(extensionregistry.loaded_extensions) == 1
    assert isinstance(extensionregistry.loaded_extensions[0], WantedExtension)


def test_extension_loading_order(extensionregistry, mocker):
    """
    Test the loading order of extensions
    """
    # given
    @extension
    class SecondExtension(object):
        LOAD_IF = staticmethod(lambda config: True)
        # default prio = 1000

    @extension
    class FirstExtension(object):
        LOAD_IF = staticmethod(lambda config: True)
        LOAD_PRIORITY = 1

    @extension
    class ThirdExtension(object):
        LOAD_IF = staticmethod(lambda config: True)
        LOAD_PRIORITY = 10000

    # when
    extensionregistry.load(mocker.MagicMock())

    # then
    assert len(extensionregistry.loaded_extensions) == 3
    assert isinstance(extensionregistry.loaded_extensions[0], FirstExtension)
    assert isinstance(extensionregistry.loaded_extensions[1], SecondExtension)
    assert isinstance(extensionregistry.loaded_extensions[2], ThirdExtension)


def test_getting_extension_options(extensionregistry, mocker):
    """
    Test getting command line options from extensions
    """
    # given
    @extension
    class FooExtension(object):
        OPTIONS = [('--foo', 'enable foo power')]

    @extension
    class BarExtension(object):
        OPTIONS = [
            ('--bar', 'enable bar power'),
            ('--bar-pow', 'enable magnitude of bar power')
        ]

    @extension
    class BlaExtension(object):
        pass

    # when
    options = extensionregistry.get_options()
    option_description = extensionregistry.get_option_description()

    # then
    assert options == """[--foo]
           [--bar]
           [--bar-pow]"""

    assert option_description == """--foo                                       enable foo power
    --bar                                       enable bar power
    --bar-pow                                   enable magnitude of bar power"""



# -*- coding: utf-8 -*-

"""
    This module provides several utility functions
"""

from __future__ import unicode_literals
from __future__ import print_function

import os
import re
import sys
import fnmatch
import traceback
import warnings
import pydoc
import itertools

from .compat import PY2, u


class Failure(object):  # pylint: disable=too-few-public-methods
    """
        Represents the fail reason for a step
    """
    def __init__(self, exception):
        """
            Initalizes the Step failure with a given Exception

            :param Exception exception: the exception shrown in the step
        """
        self.exception = exception
        self.reason = u(str(exception))
        self.traceback = u(traceback.format_exc())
        self.name = exception.__class__.__name__
        traceback_info = traceback.extract_tb(sys.exc_info()[2])[-1]
        self.filename = traceback_info[0]
        self.line = int(traceback_info[1])


def console_write(text):
    """
        Writes the given text to the console

        If the --no-colors flag is given all colors are removed from the text
    """
    if PY2 and isinstance(text, unicode):
        text = text.encode('utf-8')

    print(text)


def expandpath(path):
    """
        Expands a path

        :param string path: the path to expand
    """
    return os.path.expanduser(os.path.expandvars(path))


def recursive_glob(root, pattern):
    """
        Recursively search for files with given pattern inside a path

        :param str root: the root location to start search
        :param str pattern: to pattern to look for. It's matched against the filenames under `root`.

        :rtype: list
        :returns: A list of matching files
    """
    matches = []
    for root, dirnames, filenames in os.walk(root):
        for filename in fnmatch.filter(filenames, pattern):
            matches.append(os.path.join(root, filename))
    return matches


def get_debugger():
    """
        Returns a debugger instance
    """
    try:
        from IPython.core.debugger import Pdb
        pdb = Pdb()
    except ImportError:
        try:
            from IPython.Debugger import Pdb
            from IPython.Shell import IPShell

            IPShell(argv=[""])
            pdb = Pdb()
        except ImportError:
            warnings.warn('pdb was selected as a debugger. If you want to use ipython as a debugger you have to "pip install radish-bdd[ipython-debugger]"')
            import pdb

    return pdb


def datetime_to_str(datetime):
    """
        Returns the datetime object in a defined human readable format.

        :param Datetime datetime: the datetime object
    """
    if not datetime:
        return ""

    return datetime.strftime("%Y-%m-%dT%H:%M:%S")


def get_width(data):
    """
        Returns the needed width for a data column

        :param list data: a column with data
    """
    return max(len(x) for x in data)


def make_unique_obj_list(somelist, attr):
    """
        Make list with objects unique
        according to an objects attribute.
    """
    tmp = {}
    for item in somelist:
        tmp[attr(item)] = item
    return tmp.values()


def get_func_code(func):
    """
        Get the code object for the given function.
    """
    if sys.version_info[0] == 3:
        return func.__code__
    else:
        return func.func_code


def get_func_arg_names(func):
    """
        Get the argument names of the given function.
    """
    return get_func_code(func).co_varnames


def get_func_location(func):
    """
        Get the location where the given function is implemented.
    """
    func_code = get_func_code(func)
    return '{0}:{1}'.format(func_code.co_filename, func_code.co_firstlineno)


def str_lreplace(pattern, replacement, string, escape_pattern=False, flags=0):
    """
    Only replace the pattern with replacement
    if the string starts with the pattern.
    """
    if escape_pattern:
        pattern = re.escape(pattern)

    return re.sub(r'^{0}'.format(pattern), replacement, string, flags=flags)


def locate(name):
    """
    Locate the object for the given name
    """
    obj = pydoc.locate(name)
    if not obj:
        obj = globals().get(name, None)

    return obj


def flattened_basedirs(basedirs):
    """
    Flatten a list of basedirs.

    Multiple basedirs can be specified within a
    single element split by a colon.
    """
    return list(x for x in itertools.chain(*(x.split(':') for x in basedirs)) if x)



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import os

import pytest

from radish.terrain import world
from radish.core import Core, Configuration
from radish.parser import FeatureParser
from radish.stepregistry import StepRegistry
from radish.hookregistry import HookRegistry
from radish.extensionregistry import ExtensionRegistry

#: Holds the path to the Feature file resources
__TEST_BASE_DIR__ = os.path.dirname(__file__)
__FEATURE_FILES_DIR__ = os.path.join(__TEST_BASE_DIR__, 'features')
__RADISH_FILES_DIR__ = os.path.join(__TEST_BASE_DIR__, 'radish')
__OUTPUT_DIR__ = os.path.join(__TEST_BASE_DIR__, 'output')


@pytest.fixture(scope='function', autouse=True)
def mock_world_config():
    """
    Fixture to mock the terrain.world.config object
    with some default fake data.
    """
    # default command line arguments
    arguments = {
        '--basedir': ['$PWD/radish'],
        '--bdd-xml': None,
        '--cover-append': False,
        '--cover-branches': False,
        '--cover-config-file': '.coveragerc',
        '--cover-erase': False,
        '--cover-html': None,
        '--cover-min-percentage': None,
        '--cover-packages': None,
        '--cover-xml': None,
        '--cucumber-json': None,
        '--debug-after-failure': False,
        '--debug-steps': False,
        '--dry-run': False,
        '--early-exit': False,
        '--expand': False,
        '--help': False,
        '--inspect-after-failure': False,
        '--junit-xml' : None,
        '--marker': 'time.time()',
        '--no-ansi': False,
        '--no-line-jump': False,
        '--profile': None,
        '--scenarios': None,
        '--shuffle': False,
        '--syslog': False,
        '--tags': None,
        '--user-data': [],
        '--version': False,
        '--with-coverage': False,
        '--with-traceback': False,
        '--write-ids': False,
        '--write-steps-once': False,
        '<features>': ['features/'],
        'show': False
    }
    world.config = Configuration(arguments)
    yield world.config
    delattr(world, 'config')


@pytest.fixture(scope='function', autouse=True)
def reset_registries():
    """
    Fixture to automatically reset singleton registries
    """
    StepRegistry().clear()
    HookRegistry().reset()
    ExtensionRegistry().reset()


@pytest.fixture
def world_config(mock_world_config):
    """
    Fixture to work with world.config object
    """
    yield mock_world_config


@pytest.fixture()
def mock_utils_debugger(mocker):
    """
    Fixture to mock the pdf Python debugger
    """
    def call_orig_func(func, *args, **kwargs):
        """
        Helper to mock pdf.runcall interface
        """
        return func(*args, **kwargs)


    debugger_mock = mocker.patch('radish.utils.get_debugger')
    debugger_mock.return_value.runcall = mocker.MagicMock(side_effect=call_orig_func)
    return debugger_mock.return_value


@pytest.fixture()
def core():
    """
    Fixture to radish.core.Core
    """
    return Core()


@pytest.fixture()
def featurefile(request):
    """
    Fixture to get the path to a Feature File
    """
    featurename = request.param[0]
    return os.path.join(__FEATURE_FILES_DIR__, featurename + '.feature')


@pytest.fixture()
def featurefiledir():
    """
    Fixture to return the location of the test feature file dir
    """
    return __FEATURE_FILES_DIR__


@pytest.fixture()
def radishdir():
    """
    Fixture to return the location of the test radish files dir
    """
    return __RADISH_FILES_DIR__


@pytest.fixture()
def outputdir():
    """
    Fixture to return the location of the test output dir
    """
    return __OUTPUT_DIR__


@pytest.fixture()
def parser(request, core):
    """
    Fixture to create a Feature Parser ready to parse the given Feature File
    """
    featurename = request.param[0]
    # get parser positional arguments
    try:
        parser_args = request.param[1]
    except IndexError:
        parser_args = []
    # get parser keyword arguments
    try:
        parser_kwargs = request.param[2]
    except IndexError:
        parser_kwargs = {}

    # create the Feature Parser instance
    return FeatureParser(core, os.path.join(__FEATURE_FILES_DIR__, featurename + '.feature'),
                         1, *parser_args, **parser_kwargs)


@pytest.fixture()
def stepregistry():
    """
    Fixture to create and get a clean StepRegistry instance.
    """
    registry = StepRegistry()
    yield registry


@pytest.fixture()
def hookregistry():
    """
    Fixture to create and get a clean HookRegistry instance.
    """
    registry = HookRegistry()
    yield registry


@pytest.fixture()
def extensionregistry():
    """
    Fixture to create and get a clean ExtensionRegistry instance.
    """
    registry = ExtensionRegistry()
    yield registry



import re

from radish.stepregistry import step
from radish import when, then
from radish.terrain import world


@step(re.compile("I have the number in user data as (.+)"))
def have_number(step, input_variable):
    if world.config.user_data:
        if input_variable in world.config.user_data:
            step.context.numbers.append(int(world.config.user_data[input_variable]))
        else:
            msg = 'Variable [{0}] is not in the user data (-u/--user-data) specified on the command-line.'
            assert False, msg.format(input_variable)
    else:
        assert False, 'There is no user data (-u/--user-data) specified on the command-line.'


@when("I sum them")
def sum_numbers(step):
    step.context.result = sum(step.context.numbers)


@then(re.compile("I expect the result to be the value in user data as (.+)"))
def expect_result(step, result_variable):
    if world.config.user_data:
        if result_variable in world.config.user_data:
            assert step.context.result == int(world.config.user_data[result_variable])
        else:
            msg = 'Variable [{0}] is not in the user data (-u/--user-data) specified on the command-line.'
            assert False, msg.format(input_variable)
    else:
        assert False, 'There is no user data (-u/--user-data) specified on the command-line.'



# -*- coding: utf-8 -*-

from radish import steps

@steps
class Calculator(object):
    def have_number(self, step, number):
        """I have the number {number:g}"""
        step.context.numbers.append(number)

    def sum_numbres(self, step):
        """I sum them"""
        step.context.result = sum(step.context.numbers)

    def expect_result(self, step, result):
        """I expect the result to be {result:g}"""
        assert step.context.result == result



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

from radish.scenarioloop import ScenarioLoop
from radish.iterationscenario import IterationScenario
from radish.background import Background
from radish.stepmodel import Step


def test_creating_simple_scenarioloop():
    """
    Test creating a simple ScenarioLoop
    """
    # given & when
    scenario = ScenarioLoop(1, 'Scenario Loop', 'Iterations', 'I am a Scenario Loop', 'foo.feature', 1, parent=None,
                            tags=None, preconditions=None, background=None)

    # then
    assert scenario.id == 1
    assert scenario.keyword == 'Scenario Loop'
    assert scenario.iterations_keyword == 'Iterations'
    assert scenario.sentence == 'I am a Scenario Loop'
    assert scenario.path == 'foo.feature'
    assert scenario.line == 1
    assert scenario.parent is None
    assert scenario.tags == []
    assert scenario.preconditions == []
    assert scenario.background is None


def test_building_scenarioloop_scenarios(mocker):
    """
    Test building Scenarios from a Scenario Loop
    """
    # given
    scenario_loop = ScenarioLoop(1, 'Scenario Loop', 'Iterations', 'I am a Scenario Loop', 'foo.feature', 1, parent=None,
                                 tags=None, preconditions=None, background=None)
    # add steps
    scenario_loop.steps.extend([
        mocker.MagicMock(sentence='Given I have 1', path='foo.feature'),
        mocker.MagicMock(sentence='And I have 2', path='foo.feature'),
        mocker.MagicMock(sentence='When I add those', path='foo.feature')
    ])
    # set iterations
    scenario_loop.iterations = 2

    # when - build the scenarios
    scenario_loop.build_scenarios()

    # then - expect 2 built Scenarios
    assert len(scenario_loop.scenarios) == 2
    # then - expect that Scenarios are of type ExampleScenario
    assert all(isinstance(x, IterationScenario) for x in scenario_loop.scenarios)
    # then - expect correct Example Scenario sentences
    assert scenario_loop.scenarios[0].sentence == 'I am a Scenario Loop - iteration 0'
    assert scenario_loop.scenarios[1].sentence == 'I am a Scenario Loop - iteration 1'
    # then - expect correctly replaced Step sentences
    assert scenario_loop.scenarios[0].steps[0].sentence == 'Given I have 1'
    assert scenario_loop.scenarios[0].steps[1].sentence == 'And I have 2'
    assert scenario_loop.scenarios[0].steps[2].sentence == 'When I add those'
    assert scenario_loop.scenarios[1].steps[0].sentence == 'Given I have 1'
    assert scenario_loop.scenarios[1].steps[1].sentence == 'And I have 2'
    assert scenario_loop.scenarios[1].steps[2].sentence == 'When I add those'


def test_building_scenarioloop_scenarios_with_background(mocker):
    """
    Test building Scenarios from a Scenario Loop including a Background
    """
    # given
    background = Background('Background', 'I am a Background', 'foo.feature', 1, parent=None)
    # add some Steps
    background.steps.extend([
        Step(1, 'Foo', 'foo.feature', 2, background, False),
        Step(2, 'Foo', 'foo.feature', 3, background, False)
    ])
    scenario_loop = ScenarioLoop(1, 'Scenario Loop', 'Iterations', 'I am a Scenario Loop', 'foo.feature', 1, parent=None,
                                 tags=None, preconditions=None, background=background)
    # add steps
    scenario_loop.steps.extend([
        mocker.MagicMock(sentence='Given I have 1', path='foo.feature'),
        mocker.MagicMock(sentence='And I have 2', path='foo.feature'),
        mocker.MagicMock(sentence='When I add those', path='foo.feature')
    ])
    # set iterations
    scenario_loop.iterations = 2

    # when - build the scenarios
    scenario_loop.build_scenarios()

    # then - expect ExampleScenarios to have background copy assigned
    assert scenario_loop.scenarios[0].background.sentence == 'I am a Background'
    assert scenario_loop.scenarios[1].background.sentence == 'I am a Background'


def test_scenarioloop_afterparse_logic(mocker):
    """
    Test Scenario Loop after parse logic
    """
    # given
    scenario_loop = ScenarioLoop(1, 'Scenario Loop', 'Iterations', 'I am a Scenario Loop', 'foo.feature', 1, parent=None,
                                 tags=None, preconditions=None, background=None)
    # add steps
    scenario_loop.steps.extend([
        mocker.MagicMock(sentence='Given I have 1', path='foo.feature'),
        mocker.MagicMock(sentence='And I have 2', path='foo.feature'),
        mocker.MagicMock(sentence='When I add those', path='foo.feature')
    ])
    # set iterations
    scenario_loop.iterations = 2

    # when
    scenario_loop.after_parse()

    # then - expect 2 built Scenarios
    assert len(scenario_loop.scenarios) == 2
    assert scenario_loop.complete is True



# -*- coding: utf-8 -*-

"""
    This module provides some functionality to diagnose thrown exceptions
"""


import sys
from functools import wraps

import colorful

from .terrain import world
from .exceptions import RadishError, FeatureFileSyntaxError, StepDefinitionNotFoundError, HookError, SameStepError
from .utils import Failure, console_write


__RADISH_DOC__ = "https://github.com/radish-bdd/radish"


def write_error(text):
    """
        Writes the given text to the console
    """
    console_write("{0}: {1}".format(colorful.bold_red("Error"), colorful.red(text)))


def write_failure(failure):
    """
        Writes the failure to the console
    """
    console_write("\n{0}".format(colorful.red(failure.traceback)))


def abort(return_code):
    """
        Aborts the program with the given return_code
    """
    sys.exit(return_code)


def error_oracle(func):
    """
        Decorator to diagnose thrown exceptions
    """
    @wraps(func)
    def _decorator(*args, **kwargs):
        """
            The actual decorator
        """
        try:
            return func(*args, **kwargs)
        except Exception as e:  # pylint: disable=broad-except
            handle_exception(e)

    return _decorator


def catch_unhandled_exception(exc_type, exc_value, traceback):
    """
        Catch all unhandled exceptions
    """
    handle_exception(exc_value)


def handle_exception(exception):
    """
        Handle the given exception

        This will print more information about the given exception

        :param Exception exception: the exception to handle
    """
    if isinstance(exception, HookError):
        write_error(exception)
        write_failure(exception.failure)
        abort(1)
    elif isinstance(exception, RadishError):
        write_error(exception)
        abort(1)
    elif isinstance(exception, KeyboardInterrupt):
        console_write("Aborted by the user...")
        abort(1)
    else:
        write_error(exception)
        write_failure(Failure(exception))
        abort(2)



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""


from radish.background import Background
from radish.stepmodel import Step


def test_creating_simple_background():
    """
    Test creating a simple Background
    """
    # given & when
    background = Background('Background', 'I am a Background', 'foo.feature', 1, parent=None)

    # then
    assert background.id is None
    assert background.keyword == 'Background'
    assert background.sentence == 'I am a Background'
    assert background.path == 'foo.feature'
    assert background.line == 1
    assert background.parent is None


def test_creating_a_concrete_background_instance():
    """
    Test creating a concrete Background instance
    """
    # given & when
    background = Background('Background', 'I am a Background', 'foo.feature', 1, parent=None)
    # add some Steps
    background.steps.append(
        Step(1, 'Foo', 'foo.feature', 2, background, False)
    )
    background.steps.append(
        Step(2, 'Foo', 'foo.feature', 3, background, False)
    )

    # when
    instance = background.create_instance()

    # then
    assert len(instance.steps) == 2
    assert background.steps[0] != instance.steps[0]
    assert background.steps[1] != instance.steps[1]



# -*- coding: utf-8 -*-

from radish import given, when, then


@given('I have the number {number:g}')
def have_number(step, number):
    step.context.numbers.append(number)


@when('I sum them')
def sum_numbers(step):
    step.context.result = sum(step.context.numbers)


@then('I expect the result to be {result:g}')
def expect_result(step, result):
    assert step.context.result == result



# -*- coding: utf-8 -*-

from radish import before
from radish import custom_type, register_custom_type, TypeBuilder


@before.each_scenario
def init_numbers(scenario):
    scenario.context.numbers = []


@custom_type("Number", r'\d+')
def number_type(text):
    """
    Return the text as number
    """
    return int(text)


# create cardinalty
register_custom_type(NumberList=TypeBuilder.with_many(number_type, listsep='and'))

# choice
register_custom_type(AnswerChoice=TypeBuilder.make_choice(['yes', 'no']))



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import re

import pytest

from radish.stepregistry import step, steps
from radish.stepregistry import given, when, then
import radish.exceptions as errors


def test_registering_simple_steps(stepregistry):
    """
    Test registering simple Step functions
    """
    # given
    def step_a(): pass
    def step_b(): pass

    # when
    stepregistry.register('step_pattern_a', step_a)
    stepregistry.register('step_pattern_b', step_b)

    # then
    assert len(stepregistry.steps) == 2
    assert stepregistry.steps['step_pattern_a'] == step_a
    assert stepregistry.steps['step_pattern_b'] == step_b


def test_registering_step_pattern_twice(stepregistry):
    """
    Test registering the same Step pattern twice
    """
    # given
    def step_a(): pass
    def step_b(): pass

    stepregistry.register('step_pattern_a', step_a)

    # when
    with pytest.raises(errors.SameStepError) as exc:
        stepregistry.register('step_pattern_a', step_b)

    # then
    assert str(exc.value).startswith(
        "Cannot register step step_b with regex 'step_pattern_a' because it is already used by step step_a")


def test_registering_steps_via_object(stepregistry):
    """
    Test registering Steps via object
    """
    # given
    class MySteps(object):
        def some_step(self):
            """When I call some step"""

        def some_other_step(self):
            """
            I do some stuff

            This is not part of the step pattern
            """

    # when
    steps_object = MySteps()
    stepregistry.register_object(steps_object)

    # then
    assert len(stepregistry.steps) == 2
    assert stepregistry.steps['When I call some step'] == steps_object.some_step
    assert stepregistry.steps['I do some stuff'] == steps_object.some_other_step


def test_ignore_methods_registering_object(stepregistry):
    """
    Test ignoring methods when registering an object
    """
    # given
    class MySteps(object):
        ignore = ['some_method']

        def some_step(self):
            """When I call some step"""

        def some_method(self):
            pass

    # when
    steps_object = MySteps()
    stepregistry.register_object(steps_object)

    # then
    assert len(stepregistry.steps) == 1
    assert stepregistry.steps['When I call some step'] == steps_object.some_step


def test_error_if_no_step_regex_given_for_object(stepregistry):
    """
    Test error if a step method in object has no regex
    """
    # given
    class MySteps(object):
        def some_step(self):
            pass

    # when
    steps_object = MySteps()

    with pytest.raises(errors.RadishError) as exc:
        stepregistry.register_object(steps_object)

    # then
    assert str(exc.value) == "Step definition 'some_step' from class must have step regex in docstring"


def test_invalid_regex_step_pattern_in_method_docstring(stepregistry):
    """
    Test invalid regex Step pattern in method docstring
    """
    # given
    class MySteps(object):
        def some_step(self):
            """
            So (( invalid )(
            """

    # when
    steps_object = MySteps()

    with pytest.raises(errors.StepRegexError) as exc:
        stepregistry.register_object(steps_object)

    # then
    assert str(exc.value).startswith("Cannot compile regex 'So (( invalid )(' from step")


@pytest.mark.parametrize('pattern', [
    'I do some stuff',
    re.compile('I do some stuff')
], ids=[
    'Step with Step Pattern',
    'Step with Regex'
])
def test_registering_step_function_via_step_decorator(pattern, stepregistry):
    """
    Test registering Step function via step decorator
    """
    # given & when
    @step(pattern)
    def step_a(step): pass

    # then
    assert len(stepregistry.steps) == 1
    assert stepregistry.steps[pattern] == step_a


@pytest.mark.parametrize('pattern, expected_pattern', [
    ('I do some stuff', 'Given I do some stuff'),
    (re.compile('I do some stuff'), re.compile('Given I do some stuff'))
], ids=[
    'Step with Step Pattern',
    'Step with Regex'
])
def test_registering_step_function_via_given_decorator(pattern, expected_pattern, stepregistry):
    """
    Test registering Step function via given decorator
    """
    # given & when
    @given(pattern)
    def step_a(step): pass

    # then
    assert len(stepregistry.steps) == 1
    assert stepregistry.steps[expected_pattern] == step_a


@pytest.mark.parametrize('pattern, expected_pattern', [
    ('I do some stuff', 'When I do some stuff'),
    (re.compile('I do some stuff'), re.compile('When I do some stuff'))
], ids=[
    'Step with Step Pattern',
    'Step with Regex'
])
def test_registering_step_function_via_when_decorator(pattern, expected_pattern, stepregistry):
    """
    Test registering Step function via when decorator
    """
    # given & when
    @when(pattern)
    def step_a(step): pass

    # then
    assert len(stepregistry.steps) == 1
    assert stepregistry.steps[expected_pattern] == step_a


@pytest.mark.parametrize('pattern, expected_pattern', [
    ('I do some stuff', 'Then I do some stuff'),
    (re.compile('I do some stuff'), re.compile('Then I do some stuff'))
], ids=[
    'Step with Step Pattern',
    'Step with Regex'
])
def test_registering_step_function_via_then_decorator(pattern, expected_pattern, stepregistry):
    """
    Test registering Step function via then decorator
    """
    # given & when
    @then(pattern)
    def step_a(step): pass

    # then
    assert len(stepregistry.steps) == 1
    assert stepregistry.steps[expected_pattern] == step_a


def test_registering_steps_from_object_via_steps_decorator(stepregistry):
    """
    Test registering Steps from object via steps decorator
    """
    # given & when
    @steps
    class MySteps(object):
        def some_step(self):
            """When I call some step"""

        def some_other_step(self):
            """
            I do some stuff

            This is not part of the step pattern
            """

    # then
    assert len(stepregistry.steps) == 2
    assert 'When I call some step' in stepregistry.steps
    assert 'I do some stuff' in stepregistry.steps


def test_getting_pattern_of_specific_func(stepregistry):
    """
    Test getting the pattern of a specific func
    """
    # given
    def step_a(): pass
    def step_b(): pass
    def step_c(): pass

    stepregistry.register('step_pattern_a', step_a)
    stepregistry.register('step_pattern_b', step_b)

    # when
    pattern_a = stepregistry.get_pattern(step_a)
    pattern_b = stepregistry.get_pattern(step_b)
    pattern_c = stepregistry.get_pattern(step_c)

    # then
    assert pattern_a == 'step_pattern_a'
    assert pattern_b == 'step_pattern_b'
    assert pattern_c == 'Unknown'



# -*- coding: utf-8 -*-

"""
This module provides a representation of a Feature Background.
"""

from .scenario import Scenario
from .stepmodel import Step


class Background(Scenario):
    """
    Represents a Background
    """
    def __init__(self, keyword, sentence, path, line, parent):
        super(Background, self).__init__(None, keyword, sentence, path, line, parent)

    def create_instance(self, parent=None, steps_runable=False):
        """
        Return a copy of this Background instance.
        This enables a scenario to operate on the steps
        without influencing all the other copies.
        """
        background = Background(self.keyword, self.sentence, self.path, self.line, parent)

        for step in self.all_steps:
            # FIXME(TF): move to Step.copy
            step_copy = Step(step.id, step.sentence, step.path, step.line, parent, steps_runable, step.context_class)
            step_copy.table = step.table
            step_copy.raw_text = step.raw_text
            background.steps.append(step_copy)

        return background



# -*- coding: utf-8 -*-

"""
    This module provides the Argument-Expression Registry
"""

from singleton import singleton

from .exceptions import RadishError

from parse_type import TypeBuilder


@singleton()
class CustomTypeRegistry(object):
    """
        Registry for all custom argument expressions
    """
    def __init__(self):
        self.custom_types = {}

    def register(self, name, func):
        """
        Registers a custom type
        """
        if name in self.custom_types:
            raise RadishError("Cannot register custom type with name {0} because it already exists".format(name))

        self.custom_types[name] = func


def custom_type(name, pattern):
    """
    Decorator for custom type pattern
    """
    def _decorator(func):
        """
        Actual decorator
        """
        func.pattern = pattern
        CustomTypeRegistry().register(name, func)

        return func
    return _decorator


def register_custom_type(**kwargs):
    """
    Register the given custom types
    """
    for name, func in kwargs.items():
        CustomTypeRegistry().register(name, func)


@custom_type("MathExpression", r"[0-9 +\-*/%.e]+")
def math_expression_type(text):
    """
    Custom Type which expects a valid math expression

    :param str text: the text which was matched as math expression

    :returns: calculated float number from the math expression
    :rtype: float
    """
    return float(eval(text))


@custom_type('QuotedString', r'"(?:[^"\\]|\\.)*"')
def quoted_string_type(text):
    """
    Custom type to parse a quoted string.

    Double quotes (") have to be escaped with a
    backslash within the double quotes.
    """
    return text[1:-1]


@custom_type('Boolean',
             r'(0|1|yes|Yes|YES|y|Y|no|No|NO|n|N|true|True|TRUE|false|False|FALSE|on|On|ON|off|Off|OFF)')
def boolean_type(text):
    """
    Custom type to parse a boolean value.

    The same values are parsed as YAML does:
        http://yaml.org/type/bool.html

    Plus 0 and 1
    """
    text = text.lower()
    return text == '1' or text.startswith('y') or text == 'true' or text == 'on'



# -*- coding: utf-8 -*-

import os
import sys
import warnings
from time import time

from docopt import docopt
import colorful
import tagexpressions

from . import __VERSION__
from .core import Core
from .core import Configuration
from .loader import load_modules
from .matcher import merge_steps
from .stepregistry import StepRegistry
from .hookregistry import HookRegistry
from .runner import Runner
from .extensionregistry import ExtensionRegistry
from .exceptions import FeatureFileNotFoundError, ScenarioNotFoundError
from .errororacle import error_oracle, catch_unhandled_exception
from .terrain import world
from . import utils

# use only 8 ANSI colors
# FIXME(TF): change to true colors!
colorful.use_8_ansi_colors()


def setup_config(arguments):
    """
        Parses the docopt arguments and creates a configuration object in terrain.world
    """
    world.config = Configuration(arguments)


def show_features(core):
    """
        Show the parsed features
    """
    # set needed configuration
    world.config.write_steps_once = True
    if not sys.stdout.isatty():
        world.config.no_ansi = True

    runner = Runner(HookRegistry(), show_only=True)
    runner.start(core.features_to_run, marker="show")
    return 0


def run_features(core):
    """
        Run the parsed features

        :param Core core: the radish core object
    """
    # set needed configuration
    world.config.expand = True

    # load user's custom python files
    for basedir in utils.flattened_basedirs(world.config.basedir):
        load_modules(basedir)

    # match feature file steps with user's step definitions
    merge_steps(core.features, StepRegistry().steps)

    # run parsed features
    if world.config.marker == "time.time()":
        world.config.marker = int(time())

    # scenario choice
    amount_of_scenarios = sum(len(f.scenarios) for f in core.features_to_run)
    if world.config.scenarios:
        world.config.scenarios = [int(s) for s in world.config.scenarios.split(",")]
        for s in world.config.scenarios:
            if not 0 < s <= amount_of_scenarios:
                raise ScenarioNotFoundError(s, amount_of_scenarios)

    runner = Runner(HookRegistry(), early_exit=world.config.early_exit)
    return runner.start(core.features_to_run, marker=world.config.marker)


@error_oracle
def main(args=None):
    """
    Entrypont to radish.
    Setup up configuration, loads extensions, reads feature files and runs
    radish
    """

    if args is None:
        args = sys.argv[1:]

    # note: using doc string for usage, messes up Sphinx documantation
    usage = """
Usage:
    radish show <features>
           [--expand]
           [--no-ansi]
    radish <features>...
           [-b=<basedir> | --basedir=<basedir>...]
           [-e | --early-exit]
           [--debug-steps]
           [-t | --with-traceback]
           [-m=<marker> | --marker=<marker>]
           [-p=<profile> | --profile=<profile>]
           [-d | --dry-run]
           [-s=<scenarios> | --scenarios=<scenarios>]
           [--shuffle]
           [--tags=<tags>]
           {0}
    radish (-h | --help)
    radish (-v | --version)

Arguments:
    features                                    feature files to run

Options:
    -h --help                                   show this screen
    -v --version                                show version
    -e --early-exit                             stop the run after the first failed step
    --debug-steps                               debugs each step
    -t --with-traceback                         show the Exception traceback when a step fails
    -m=<marker> --marker=<marker>               specify the marker for this run [default: time.time()]
    -p=<profile> --profile=<profile>            specify the profile which can be used in the step/hook implementation
    -b=<basedir> --basedir=<basedir>...         set base dir from where the step.py and terrain.py will be loaded. [default: $PWD/radish]
                                                You can specify -b|--basedir multiple times or split multiple paths with a colon (:) similar to $PATH. All files will be imported.
    -d --dry-run                                make dry run for the given feature files
    -s=<scenarios> --scenarios=<scenarios>      only run the specified scenarios (comma separated list)
    --shuffle                                   shuttle run order of features and scenarios
    --tags=<feature_tags>                       only run Scenarios with the given tags
    --expand                                    expand the feature file (all preconditions)
    {1}

(C) Copyright by Timo Furrer <tuxtimo@gmail.com>
    """

    warnings.simplefilter('always', DeprecationWarning)

    # load extensions
    load_modules(os.path.join(os.path.dirname(__file__), "extensions"))

    extensions = ExtensionRegistry()
    # add arguments from extensions to the usage
    usage = usage.format(extensions.get_options(), extensions.get_option_description())

    sys.excepthook = catch_unhandled_exception

    # add version to the usage
    arguments = docopt("radish {0}\n{1}".format(__VERSION__, usage), argv=args, version=__VERSION__)

    # store all arguments to configuration dict in terrain.world
    setup_config(arguments)

    # disable colors if necessary
    if world.config.no_ansi:
        colorful.disable()
    else:
        colorful.use_8_ansi_colors()

    # load needed extensions
    extensions.load(world.config)

    core = Core()

    if world.config.profile:
        msg = ('Command line argument -p/--profile will be removed in a future version.  Please '
               'use -u/--user-data instead.')
        warnings.warn(msg, DeprecationWarning, stacklevel=1)

    feature_files = []
    for given_feature in world.config.features:
        if not os.path.exists(given_feature):
            raise FeatureFileNotFoundError(given_feature)

        if os.path.isdir(given_feature):
            feature_files.extend(utils.recursive_glob(given_feature, "*.feature"))
            continue

        feature_files.append(given_feature)

    # parse tag expressions
    tag_expression = None
    if world.config.tags:
        tag_expression = tagexpressions.parse(world.config.tags)

    core.parse_features(feature_files, tag_expression)

    if not core.features or sum(len(f.scenarios) for f in core.features) == 0:
        print(colorful.bold_red('Error: ') + colorful.red('please specify at least one feature to run'))
        if tag_expression:
            print(colorful.red('You have specified a tag expression. Make sure those are valid and actually yield some Scenarios to run.'))
        return 1

    argument_dispatcher = [
        ((lambda: world.config.show), show_features),
        ((lambda: True), run_features)
    ]

    # radish command dispatching
    for to_run, method in argument_dispatcher:
        if to_run():
            return method(core)


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-

"""
    This module provides a class to represent a Scenario Outline
"""

import copy

from .scenario import Scenario
from .examplescenario import ExampleScenario
from .stepmodel import Step
from .exceptions import RadishError


class ScenarioOutline(Scenario):
    """
        Represents a Scenario
    """

    class Example(object):
        """
            Represents the ScenarioOutline examples
        """

        def __init__(self, data, path, line):
            self.data = data
            self.path = path
            self.line = line

    def __init__(self, id, keyword, example_keyword, sentence, path, line, parent, tags=None, preconditions=None, background=None):
        super(ScenarioOutline, self).__init__(
                id, keyword, sentence, path, line, parent,
                tags, preconditions, background)
        self.example_keyword = example_keyword
        self.scenarios = []
        self.examples_header = []
        self.examples = []

    def build_scenarios(self):
        """
            Builds the scenarios with the parsed Examples

            Note: This must be done before mering the steps
                  from the feature file with the step definitions
        """
        for row_id, example in enumerate(self.examples):
            examples = dict(zip(self.examples_header, example.data))
            scenario_id = self.id + row_id + 1
            background = None
            scenario = ExampleScenario(
                    scenario_id, self.keyword,
                    "{0} - row {1}".format(
                        self.sentence, row_id), self.path, self.line, self, example)
            if self.background:
                background = self.background.create_instance(parent=scenario, steps_runable=True)
                scenario.background = background

            for step_id, outlined_step in enumerate(self.steps):
                sentence = self._replace_examples_in_sentence(outlined_step.sentence, examples)
                step = Step(
                        step_id + 1, sentence, outlined_step.path, example.line,
                        scenario, True, context_class=outlined_step.context_class)
                # copy extended attributes
                step.table_header = copy.copy(outlined_step.table_header)
                step.table_data = copy.copy(outlined_step.table_data)
                step.table = copy.copy(outlined_step.table)
                step.raw_text = copy.copy(outlined_step.raw_text)

                # add step to scenario
                scenario.steps.append(step)
            self.scenarios.append(scenario)

    @staticmethod
    def _replace_examples_in_sentence(sentence, examples):
        """
            Replaces the given examples in the given sentece

            :param string sentence: the step sentence in which to replace the examples
            :param dict examples: the examples

            :returns: the new step sentence
            :rtype: string
        """
        for key, value in examples.items():
            sentence = sentence.replace("<{0}>".format(key), value)
        return sentence

    def get_column_width(self, column_index):
        """
            Gets the column width from the given column

            :param int column_index: the column index to get the width from
        """
        try:
            return max(max([len(x.data[column_index]) for x in self.examples]), len(self.examples_header[column_index]))
        except IndexError:
            raise RadishError("Invalid colum_index to get column width for ScenarioOutline '{0}'".format(self.sentence))

    def after_parse(self):
        """
            Build outlined scenarios
        """
        Scenario.after_parse(self)
        self.build_scenarios()
        self.complete = True



# -*- coding: utf-8 -*-

from radish import before


@before.each_scenario
def before_scenario(scenario):
    # print(scenario.context)
    scenario.context.users = {}
    scenario.context.current_user = None



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

from __future__ import unicode_literals

import pytest

import tagexpressions

from radish.core import Core
from radish.parser import FeatureParser
from radish.model import Tag
from radish.scenariooutline import ScenarioOutline
from radish.scenarioloop import ScenarioLoop
from radish.background import Background
import radish.exceptions as errors


@pytest.mark.parametrize('parser', [
    ('empty-feature',)
], indirect=['parser'])
def test_parse_invalid_state(parser):
    """
    Test invalid state when parsing
    """
    # given
    parser._current_state = 'not_existing_state'

    # when
    with pytest.raises(errors.RadishError) as exc:
        parser.parse()

    # then
    assert str(exc.value) == 'FeatureParser state not_existing_state is not supported'


@pytest.mark.parametrize('language, feature_keyword, scenario_keyword', [
    ('en', 'Feature', 'Scenario'),
    ('de', 'Funktionalität', 'Szenario')
])
def test_creating_language_agnostic_parser(language, feature_keyword, scenario_keyword, core):
    """
    Test creating a language agnostic parser
    """
    # given & when
    parser = FeatureParser(core, '/', 1, language=language)

    # then
    assert parser.keywords.feature == feature_keyword
    assert parser.keywords.scenario == scenario_keyword


def test_creating_parser_for_not_supported_lang(core):
    """
    Test creating a Parser for a not supported language
    """
    # given & when
    with pytest.raises(errors.LanguageNotSupportedError) as exc:
        FeatureParser(core, '/', 1, language='chruesimuesi')

    # then
    assert str(exc.value) == 'Language chruesimuesi could not be found'


def test_creating_parser_with_unsupported_lang_detection(core):
    """
    Test creating a Parser with yet unsupported language auto detection
    """
    # given & when
    with pytest.raises(NotImplementedError) as exc:
        FeatureParser(core, '/', 1, language=None)

    # then
    assert str(exc.value) == 'Auto detect language is not implemented yet'


@pytest.mark.parametrize('parser', [
    ('german',)
], indirect=['parser'])
def test_parse_featurefile_with_language_tag(parser):
    """
    Test parsing a Feature File with a language tag
    """
    # when
    parser.parse()

    # then
    assert parser.feature.sentence == 'Das ist eine radish Funktionalität'
    assert parser.feature.scenarios[0].sentence == 'Dies ist ein Szenario'


def test_parse_not_existing_featurefile(core):
    """
    Test parsing a not existing Feature File
    """
    # given & when
    with pytest.raises(OSError) as exc:
        FeatureParser(core, 'does-not-exist.feature', 1)

    # then
    assert str(exc.value) == "Feature file at 'does-not-exist.feature' does not exist"


@pytest.mark.parametrize('parser', [
    ('empty',)
], indirect=['parser'])
def test_parse_empty_featurefile(parser):
    """
    Test parsing an empty Feature File
    """
    # when
    with pytest.raises(errors.FeatureFileSyntaxError) as exc:
        parser.parse()

    # then
    assert str(exc.value).startswith('No Feature found in file')


@pytest.mark.parametrize('parser', [
    ('empty-feature',)
], indirect=['parser'])
def test_parse_empty_feature(parser):
    """
    Test parsing an empty Feature
    """
    # when
    parser.parse()

    # then
    assert parser.feature.sentence == 'This is an empty Feature'
    assert len(parser.feature.scenarios) == 0


@pytest.mark.parametrize('parser', [
    ('feature-only-description',)
], indirect=['parser'])
def test_parse_empty_feature_with_description(parser):
    """
    Test parsing a Feature with Description but without Scenarios
    """
    # when
    parser.parse()

    # then
    assert parser.feature.description[0] == 'This is a description'
    assert parser.feature.description[1] == 'with two lines'


@pytest.mark.parametrize('parser', [
    ('multi-features',)
], indirect=['parser'])
def test_parse_featurefile_with_multiple_features(parser):
    """
    Test parsing a Feature File containing multiple Features
    """
    # when
    with pytest.raises(errors.RadishError) as exc:
        parser.parse()

    # then
    assert str(exc.value).startswith('radish supports only one Feature per feature file')


@pytest.mark.parametrize('parser', [
    ('syntax-error-unknown-keyword',)
], indirect=['parser'])
def test_parse_featurefile_with_syntax_error(parser):
    """
    Test parsing a Feature File with Syntax Errors
    """
    # when
    with pytest.raises(errors.FeatureFileSyntaxError) as exc:
        parser.parse()

    # then
    assert str(exc.value).startswith('Syntax error in feature file')


@pytest.mark.parametrize('parser', [
    ('empty-scenario',)
], indirect=['parser'])
def test_parse_feature_with_empty_scenario(parser):
    """
    Test parsing a Feature with an empty Scenario
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == 1
    assert feature.scenarios[0].sentence == 'This is an empty Scenario'
    assert len(feature.scenarios[0].steps) == 0


@pytest.mark.parametrize('parser', [
    ('feature-scenario-steps',)
], indirect=['parser'])
def test_parse_feature_with_scenario_and_steps(parser):
    """
    Test parsing a Feature with a Scenario and Steps
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == 1
    assert len(feature.scenarios[0].steps) == 3
    assert feature.scenarios[0].steps[0].sentence == 'Given I have a Step'
    assert feature.scenarios[0].steps[1].sentence == 'When I do something'
    assert feature.scenarios[0].steps[2].sentence == 'Then I expect something'


@pytest.mark.parametrize('parser', [
    ('feature-scenarios',)
], indirect=['parser'])
def test_parse_feature_with_scenarios(parser):
    """
    Test parsing a Feature with multiple Scenarios and Steps
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == 2
    assert len(feature.scenarios[0].steps) == 3
    assert len(feature.scenarios[1].steps) == 3


@pytest.mark.parametrize('parser', [
    ('comments',)
], indirect=['parser'])
def test_parse_featurefile_with_comments(parser):
    """
    Test parsing a Feature File with comments
    """
    # when
    feature = parser.parse()

    # then
    assert feature.sentence == 'Ignore comments in Parser'
    assert len(feature.description) == 1
    assert feature.description[0] == 'Radish shall ignore comments in Feature Files'
    assert len(feature.scenarios) == 1
    assert len(feature.scenarios[0].steps) == 1
    assert feature.scenarios[0].steps[0].sentence == 'When I do something'


@pytest.mark.parametrize('parser', [
    ('scenario-outline',)
], indirect=['parser'])
def test_parse_feature_with_scenario_outline(parser):
    """
    Test parsing a Feature with a simple Scenario Outline
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == 1
    assert isinstance(feature.scenarios[0], ScenarioOutline)
    assert len(feature.scenarios[0].scenarios) == 2

    # then - expect correct scenario steps with replaced fields
    assert feature.scenarios[0].scenarios[0].steps[0].sentence == 'Given I have the number 1'
    assert feature.scenarios[0].scenarios[0].steps[1].sentence == 'And I have the number 2'
    assert feature.scenarios[0].scenarios[0].steps[2].sentence == 'When I add them up'
    assert feature.scenarios[0].scenarios[0].steps[3].sentence == 'Then I expect the sum to be 3'

    assert feature.scenarios[0].scenarios[1].steps[0].sentence == 'Given I have the number 4'
    assert feature.scenarios[0].scenarios[1].steps[1].sentence == 'And I have the number 5'
    assert feature.scenarios[0].scenarios[1].steps[2].sentence == 'When I add them up'
    assert feature.scenarios[0].scenarios[1].steps[3].sentence == 'Then I expect the sum to be 9'


@pytest.mark.parametrize('parser', [
    ('scenario-outline-step-text',)
], indirect=['parser'])
def test_parse_scenario_outline_with_step_text(parser):
    """
    Test parsing a Scenario Outline with a Step with Text
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == 1
    assert isinstance(feature.scenarios[0], ScenarioOutline)
    assert len(feature.scenarios[0].scenarios) == 3

    # then - expect correct scenario steps with replaced fields
    assert feature.scenarios[0].scenarios[0].steps[0].sentence == 'Given I have the number 10'
    assert feature.scenarios[0].scenarios[0].steps[1].sentence == 'And I have the text'
    assert feature.scenarios[0].scenarios[0].steps[1].text == 'foobar'


@pytest.mark.parametrize('parser', [
    ('regular-scenario-examples',)
], indirect=['parser'])
def test_parse_feature_with_scenario_and_examples(parser):
    """
    Test parsing a Feature with a Scenario combined with Examples without a Scenario Outline
    """
    # when
    with pytest.raises(errors.RadishError) as exc:
        parser.parse()

    # then
    assert str(exc.value).startswith("Scenario does not support Examples. Use 'Scenario Outline'")


@pytest.mark.parametrize('parser', [
    ('scenario-loop',)
], indirect=['parser'])
def test_parse_feature_with_scenario_loop(parser):
    """
    Test parsing a Feature with a simple Scenario Loop
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == 1
    assert isinstance(feature.scenarios[0], ScenarioLoop)
    assert len(feature.scenarios[0].scenarios) == 2

    assert feature.scenarios[0].scenarios[0].steps[0].sentence == 'Given I have an instable function'
    assert feature.scenarios[0].scenarios[0].steps[1].sentence == 'When I execute it'
    assert feature.scenarios[0].scenarios[0].steps[2].sentence == 'Then I expect it to pass'

    assert feature.scenarios[0].scenarios[1].steps[0].sentence == 'Given I have an instable function'
    assert feature.scenarios[0].scenarios[1].steps[1].sentence == 'When I execute it'
    assert feature.scenarios[0].scenarios[1].steps[2].sentence == 'Then I expect it to pass'


@pytest.mark.parametrize('parser', [
    ('step-tabular-data',)
], indirect=['parser'])
def test_parse_step_tabular_data(parser):
    """
    Test parsing a Feature with a Scenario and Step with Tabular Data
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios[0].steps) == 3
    assert len(feature.scenarios[0].steps[0].table_header) == 3
    assert feature.scenarios[0].steps[0].table_header == ['firstname', 'surname', 'heroname']
    assert len(feature.scenarios[0].steps[0].table) == 2
    assert feature.scenarios[0].steps[0].table[0] == {
            'firstname': 'Bruce', 'surname': 'Wayne', 'heroname': 'Batman'}
    assert feature.scenarios[0].steps[0].table[1] == {
            'firstname': 'Peter', 'surname': 'Parker', 'heroname': 'Spiderman'}


@pytest.mark.parametrize('parser', [
    ('step-tabular-data-invalid',)
], indirect=['parser'])
def test_parse_step_tabular_data_invalid(parser):
    """
    Test parsing a Step Table without a Step
    """
    # when
    with pytest.raises(errors.FeatureFileSyntaxError) as exc:
        parser.parse()

    # then
    assert str(exc.value).startswith('Found step table without previous step definition on line 6')


@pytest.mark.parametrize('parser', [
    ('step-text-data',)
], indirect=['parser'])
def test_parse_step_text_data(parser):
    """
    Test parsing a Feature with a Scenario and Step with Text Data
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios[0].steps) == 3
    assert feature.scenarios[0].steps[0].text == 'To be or not to be'


@pytest.mark.parametrize('parser, expected_feature_tags, expected_scenarios_tags', [
    (['tags-feature'], [Tag('foo'), Tag('bar')], [[]]),
    (['tags-scenario'], [], [[Tag('foo'), Tag('bar')]]),
    (['tags-everywhere'], [Tag('foo'), Tag('bar')], [
        [Tag('regular_scenario')], [Tag('scenario_outline')], [Tag('scenario_loop')]
    ]),
    (['tags-arguments'], [Tag('foo', 'bar')], [[Tag('sometag', 'somevalue')]])
], indirect=['parser'])
def test_parse_tags(parser, expected_feature_tags, expected_scenarios_tags):
    """
    Test parsing Feature and Scenario Tags
    """
    # when
    feature = parser.parse()

    # then
    assert feature.tags == expected_feature_tags
    assert len(feature.scenarios) == len(expected_scenarios_tags)
    for scenario, expected_scenario_tags in zip(feature.scenarios, expected_scenarios_tags):
        assert scenario.tags == expected_scenario_tags


@pytest.mark.parametrize('parser', [
    ('tags-ignored-feature', [], {'tag_expr': tagexpressions.parse('not foo')})
], indirect=['parser'])
def test_parse_ignored_feature_via_tag(parser):
    """
    Test parsing a Feature which is ignored because of a Tag
    """
    # when
    feature = parser.parse()

    # then
    assert feature is None


@pytest.mark.parametrize('parser', [
    ('tags-ignored-scenario', [], {'tag_expr': tagexpressions.parse('not foo')})
], indirect=['parser'])
def test_parse_ignored_scenario_via_tag(parser):
    """
    Test parsing a Feature with ignored Scenarios because of a Tag
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == 2
    assert feature.scenarios[0].sentence == 'Parsed Scenario'
    assert feature.scenarios[1].sentence == 'Another parsed Scenario'


@pytest.mark.parametrize('parser', [
    ('tags-no-feature', [], {'tag_expr': tagexpressions.parse('not foo')})
], indirect=['parser'])
def test_parse_ignored_feature_empty_featurefile(parser):
    """
    Test detecting if no Feature left because of given Tag expression
    """
    # when
    feature = parser.parse()

    # then
    assert feature is None


@pytest.mark.parametrize('parser, expected_scenarios', [
    (
        ('tag-inheritance', [], {'tag_expr': tagexpressions.parse('some_feature')}), 3
    ),
    (
        ('tag-inheritance', [], {'tag_expr': tagexpressions.parse('some_feature and good_case')}), 2
    ),
    (
        ('tag-inheritance', [], {'tag_expr': tagexpressions.parse('some_feature and bad_case')}), 1
    ),
    (
        ('tag-inheritance', [], {'tag_expr': tagexpressions.parse('some_feature and not bad_case')}), 2
    ),
    (
        ('tag-inheritance', [], {'tag_expr': tagexpressions.parse('good_case or bad_case')}), 3
    ),
], indirect=['parser'])
def test_parse_tag_inheritance(parser, expected_scenarios):
    """
    Test parsing a Feature and Scenarios with Tags and check the Tag inheritance
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == expected_scenarios


@pytest.mark.parametrize('parser', [
    ('background',)
], indirect=['parser'])
def test_parse_simple_background(parser):
    """
    Test parsing a Feature with a simple Background
    """
    # when
    feature = parser.parse()

    # then
    assert isinstance(feature.background, Background)
    assert len(feature.scenarios) == 2
    assert all(s.background.sentence == feature.background.sentence for s in feature.scenarios)

@pytest.mark.parametrize('parser', [
    ('background-no-sentence',)
], indirect=['parser'])
def test_parse_background_no_sentence(parser):
    """
    Test parsing a Feature with unnamed Background
    """
    # when
    feature = parser.parse()

    # then
    assert isinstance(feature.background, Background)
    assert feature.background.sentence == ''
    assert len(feature.scenarios) == 2
    assert all(s.background.sentence == feature.background.sentence for s in feature.scenarios)


@pytest.mark.parametrize('parser', [
    ('background-scenariooutline',)
], indirect=['parser'])
def test_parse_background_for_scenario_outline(parser):
    """
    Test parsing a Feature with a Background and Scenario Outline
    """
    # when
    feature = parser.parse()

    # then
    assert isinstance(feature.background, Background)
    assert all(s.background.sentence == feature.background.sentence for s in feature.scenarios[0].scenarios)


@pytest.mark.parametrize('parser', [
    ('background-scenarioloop',)
], indirect=['parser'])
def test_parse_background_for_scenario_loop(parser):
    """
    Test parsing a Feature with a Background and Scenario Loop
    """
    # when
    feature = parser.parse()

    # then
    assert isinstance(feature.background, Background)
    assert all(s.background.sentence == feature.background.sentence for s in feature.scenarios[0].scenarios)


@pytest.mark.parametrize('parser', [
    ('background-subsequent-tag',)
], indirect=['parser'])
def test_parse_background_subsequent_tag(parser):
    """
    Test parsing a Feature with Background followed by a tagged Scenario
    """
    # when
    feature = parser.parse()

    # then
    assert isinstance(feature.background, Background)
    assert len(feature.background.steps) == 2
    assert len(feature.scenarios[0].tags) == 2


@pytest.mark.parametrize('parser', [
    ('background-misplaced',)
], indirect=['parser'])
def test_parse_misplaced_background(parser):
    """
    Test detecting a misplaced Background
    """
    # when
    with pytest.raises(errors.FeatureFileSyntaxError) as exc:
        parser.parse()

    # then
    assert str(exc.value).startswith('The Background block must be placed before any Scenario block')


@pytest.mark.parametrize('parser', [
    ('background-multiple',)
], indirect=['parser'])
def test_parse_multiple_background(parser):
    """
    Test detecting a multiple Backgrounds
    """
    # when
    with pytest.raises(errors.FeatureFileSyntaxError) as exc:
        parser.parse()

    # then
    assert str(exc.value).startswith('The Background block may only appear once in a Feature')


@pytest.mark.parametrize('parser', [
    ('constants',)
], indirect=['parser'])
def test_parse_constants(parser):
    """
    Test parsing Feature and Scenario constants
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.constants) == 1
    assert len(feature.scenarios[0].constants) == 2
    assert len(feature.scenarios[1].constants) == 2


@pytest.mark.parametrize('parser', [
    ('scenario-sentence-duplicate',)
], indirect=['parser'])
def test_parse_scenario_sentence_duplicate(parser):
    """
    Test detecting Scenario sentence duplicates
    """
    # when
    with pytest.raises(errors.FeatureFileSyntaxError) as exc:
        parser.parse()

    # then
    assert str(exc.value).startswith(
        "Scenario with name 'A great name for a Scenario' "
        "defined twice in feature")


@pytest.mark.parametrize('parser', [
    ('precondition-level-2',)
], indirect=['parser'])
def test_parse_simple_layered_preconditions(parser):
    """
    Test parsing simple three layered Scenario preconditions
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == 1
    assert len(feature.scenarios[0].preconditions) == 1
    assert len(feature.scenarios[0].preconditions[0].preconditions) == 1


@pytest.mark.parametrize('parser', [
    ('precondition-same-feature',)
], indirect=['parser'])
def test_parse_scenario_precondition_same_feature(parser):
    """
    Test parsing simple three layered Scenario preconditions
    """
    # when
    feature = parser.parse()

    # then
    assert len(feature.scenarios) == 2
    assert len(feature.scenarios[1].preconditions) == 1


@pytest.mark.parametrize('parser, expected_error_msg', [
    (
        ('precondition-unknown-scenario',), "Cannot import precondition scenario 'Unknown Scenario' from feature"
    ),
    (
        ('precondition-unknown-scenario-same-feature',), "Cannot import precondition scenario 'Unknown Scenario' from feature"
    ),
    (
        ('precondition-recursion',), 'Your feature'
    ),
    (
        ('precondition-malformed',), "Scenario @precondition tag must have argument in format: 'test.feature: Some scenario'"
    )
], ids=[
    'Unknown Scenario',
    'Unknown Scenario within same Feature',
    'Recursion',
    'Malformed precondition Tag'
], indirect=['parser'])
def test_parse_scenario_precondition_errors(parser, expected_error_msg):
    """
    Test detecting Scenario Precondition Parsing errors
    """
    # when
    with pytest.raises(errors.FeatureFileSyntaxError) as exc:
        parser.parse()

    # then
    assert str(exc.value).startswith(expected_error_msg)



# -*- coding: utf-8 -*-

"""
    Providing radish core functionality.
"""

from threading import Lock
from collections import OrderedDict

from .parser import FeatureParser


class Configuration(object):
    """
        Manage configuration. Attributes of the class are created from the
        names of the command line options and are set to the command line
        values.

        Attribute names are parsed from command-line options removing or
        replacing characters that can not be used in python variables.

        Specifically:
          * "-" is replaced with "_"
          * "--" is removed.
          * "<" and ">" are removed (they are used in positional arguments)

        :param arguments: command line arguments and their values
        :type arguments: dict-line object (i.e. docopt.Dict)
    """

    def __init__(self, arguments):
        for key, value in arguments.items():
            config_key = key.replace("--", "")\
                            .replace("-", "_")\
                            .replace("<", "")\
                            .replace(">", "")
            setattr(self, config_key, value)


# FIXME: rename
class Core(object):
    """
        Provide some core functionalities like parsing and storing of the feature files
    """
    def __init__(self):
        self.features = []
        self._features_to_run = OrderedDict()
        self._feature_id_lock = Lock()
        self._feature_id = 0
        self._scenario_id_lock = Lock()
        self._scenario_id = 0

    @property
    def features_to_run(self):
        """
            Return all parsed features which are to run
        """
        return [f for f in self._features_to_run.values()]

    @property
    def next_feature_id(self):
        """
            Returns the next feature id
        """
        with self._feature_id_lock:
            self._feature_id += 1
            return self._feature_id

    @property
    def next_scenario_id(self):
        """
            Returns the next scenario id
        """
        with self._scenario_id_lock:
            self._scenario_id += 1
            return self._scenario_id

    def parse_features(self, feature_files, tag_expr):
        """
            Parses the given feature files
        """
        for featurefile in feature_files:
            feature = self.parse_feature(featurefile, tag_expr, featureid=self.next_feature_id)

            if feature is not None:
                for scenario in feature.scenarios:
                    scenario.absolute_id = self.next_scenario_id

                self._features_to_run[featurefile] = feature

    def parse_feature(self, featurefile, tag_expr, inherited_tags=None, featureid=0):
        """
            Parses the given feature file
            If the feature is alreay parsed then it will just return it

            :returns: the parsed feature
            :rtype: Feature
        """
        featureparser = FeatureParser(self, featurefile, featureid, tag_expr, inherited_tags=inherited_tags)
        feature = featureparser.parse()

        if feature is None:
            return None

        self.features.append(feature)
        return feature



# -*- coding: utf-8 -*-

from radish import before

@before.each_scenario
def init_numbers(scenario):
    scenario.context.numbers = []


@before.each_scenario(on_tags='headstart')
def init_numbers_headstart(scenario):
    scenario.context.numbers.append(5)



# -*- coding: utf-8 -*-

import re
from radish.stepregistry import step
from radish import given, when, then

@step(re.compile(r"I have the number ([0-9]+)"))
def have_number(step, number):
    step.context.numbers.append(float(number))

@when(re.compile(r"I sum them"))
def sum_numbers(step):
    step.context.result = sum(step.context.numbers)

@then(re.compile(r"I expect the result to be (\d+)"))
def expect_result(step, result):
    assert step.context.result == float(result)



# -*- coding: utf-8 -*-

"""
    This module provides a class to represent a Feature from a parsed feature file.
"""

from .model import Model
from .scenariooutline import ScenarioOutline
from .scenarioloop import ScenarioLoop
from .stepmodel import Step
from .terrain import world


class Feature(Model):
    """
        Represent a Feature
    """

    def __init__(self, id, keyword, sentence, path, line, tags=None):
        super(Feature, self).__init__(id, keyword, sentence, path, line, None, tags)
        self.description = []
        self.background = None
        self.scenarios = []
        self.context = self.Context()

    @property
    def all_scenarios(self):
        """
            Returns all scenarios from the feature
            The ScenarioOutline scenarios will be extended to the normal scenarios
        """
        scenarios = []
        for scenario in self.scenarios:
            scenarios.append(scenario)
            if isinstance(scenario, (ScenarioOutline, ScenarioLoop)):
                scenarios.extend(scenario.scenarios)
        return scenarios

    @property
    def constants(self):
        """
            Returns all constants
        """
        return self.context.constants

    def __str__(self):
        return "Feature: {0} from {1}:{2}".format(self.sentence, self.path, self.line)

    def __repr__(self):
        return "<Feature: {0} from {1}:{2}>".format(self.sentence, self.path, self.line)

    def __iter__(self):
        """
            Returns an iterator for the scenario of this feature
        """
        return iter(self.scenarios)

    def __contains__(self, sentence):
        """
            Checks if the given scenario sentence is from a scenario of this feature

            :param str sentence: the scenario sentence to search
        """
        return any(s for s in self.scenarios if s.sentence == sentence)

    def __getitem__(self, sentence):
        """
            Returns the scenario with the given sentence

            :param str sentence: the scenario sentence to search
        """
        return next((s for s in self.scenarios if s.sentence == sentence), None)

    @property
    def state(self):
        """
            Returns the state of the scenario
        """
        for scenario in self.all_scenarios:
            if isinstance(scenario, (ScenarioOutline, ScenarioLoop)):  # skip scenario outlines
                continue

            if not scenario.has_to_run(world.config.scenarios):
                continue

            if scenario.state is not Step.State.PASSED:
                return scenario.state
        return Step.State.PASSED

    def has_to_run(self, scenario_choice):
        """
            Returns wheiter the feature has to run or not
        """
        if not scenario_choice:
            return True

        in_choice = False
        if scenario_choice:
            in_choice = any(s for s in self.scenarios if s.absolute_id in scenario_choice)

        return in_choice



# -*- coding: utf-8 -*-

"""
    This module provides a hook which processes user specified data on the command-line
    and provides this data as a dictionary attached to the world.config object
    as per Enhancement #124.
"""

import re

from radish.terrain import world
from radish.exceptions import RadishError
from radish.extensionregistry import extension


@extension
class UserData(object):
    """
        User Data radish extension
    """
    OPTIONS = [("-u=<userdata> | --user-data=<userdata>...",
               "User data as 'key=value' pair. You can specify --user-data multiple times.")
               ]
    LOAD_IF = staticmethod(lambda config: True)
    LOAD_PRIORITY = 2  # This should probably load early in-case another extension needs to inspect this data.

    def __init__(self):
        self._kv_regex = re.compile(r'\s*=\s*')

        self._cli_user_data = world.config.user_data  # Save the data to process later
        world.config.user_data = {}  # Initialize user data dictionary

        self.process_user_data()

    def process_user_data(self):
        """
            Process the user data
        """
        if self._cli_user_data:
            for pair in self._cli_user_data:
                try:
                    key, value = map(str.strip, self._kv_regex.split(pair, maxsplit=1))
                except ValueError as e:
                    msg = "-u/--user-data: User data is invalid. Expecting: 'key=value' Got: '{0}'"
                    raise RadishError(msg.format(pair))

                # Allowing all keys and values, even if empty
                world.config.user_data[key] = value



# -*- coding: utf-8 -*-

"""
    Providing radish core functionality like the feature file Runner.
"""

from random import shuffle

from .terrain import world
from .scenariooutline import ScenarioOutline
from .scenarioloop import ScenarioLoop
from .stepmodel import Step


class Runner(object):
    """
        Represents a class which is able to run features.
    """

    def handle_exit(func):  # pylint: disable=no-self-argument
        """
            Handles an runner exit
        """
        def _decorator(self, *args, **kwargs):
            """
                Actual decorator
            """
            if self._required_exit:  # pylint: disable=protected-access
                return 1

            return func(self, *args, **kwargs)  # pylint: disable=not-callable
        return _decorator

    def call_hooks(model):  # pylint: disable=no-self-argument
        """
            Call hooks for a specific model
        """
        def _decorator(func):
            """
                The actual decorator
            """
            def _wrapper(self, model_instance, *args, **kwargs):
                """
                    Decorator wrapper
                """
                self._hooks.call("before", model, model_instance, *args, **kwargs)  # pylint: disable=protected-access
                try:
                    return func(self, model_instance, *args, **kwargs)
                finally:
                    self._hooks.call("after", model, model_instance, *args, **kwargs)  # pylint: disable=protected-access
            return _wrapper
        return _decorator

    def __init__(self, hooks, early_exit=False, show_only=False):
        self._hooks = hooks
        self._early_exit = early_exit
        self._required_exit = False
        self._show_only = show_only

    @handle_exit
    @call_hooks("all")
    def start(self, features, marker):
        """
            Start running features

            :param list features: the features to run
            :param string marker: the marker for this run
        """
        if world.config.shuffle:
            shuffle(features)

        returncode = 0  # return code is set to 1 if any feature fails
        for feature in features:
            if not feature.has_to_run(world.config.scenarios):
                continue

            returncode |= self.run_feature(feature)

        return returncode

    @handle_exit
    @call_hooks("each_feature")
    def run_feature(self, feature):
        """
            Runs the given feature

            :param Feature feature: the feature to run
        """
        if world.config.shuffle:
            shuffle(feature.scenarios)

        returncode = 0
        for scenario in feature:
            if not scenario.has_to_run(world.config.scenarios):
                continue
            returncode |= self.run_scenario(scenario)

            if not isinstance(scenario, (ScenarioOutline, ScenarioLoop)):
                continue

            for sub_scenario in scenario.scenarios:
                returncode |= self.run_scenario(sub_scenario)
        return returncode

    @handle_exit
    @call_hooks("each_scenario")
    def run_scenario(self, scenario):
        """
            Runs the given scenario

            :param Scenario scenario: the scnenario to run
        """
        returncode = 0
        steps = scenario.all_steps if world.config.expand else scenario.steps
        for step in steps:
            if scenario.state == Step.State.FAILED:
                self.skip_step(step)
                continue

            returncode |= self.run_step(step)

            if step.state == step.State.FAILED and self._early_exit:
                self.exit()
                return 1
        return returncode

    @handle_exit
    @call_hooks("each_step")
    def run_step(self, step):
        """
            Runs the given step

            :param Step step: the step to run
        """
        if self._show_only:
            return 0

        if world.config.debug_steps:
            state = step.debug()
        else:
            state = step.run()

        return 1 if state == Step.State.FAILED else 0

    def skip_step(self, step):
        """
            Skips the given step

            :param Step step: the step to skip
        """
        self._hooks.call("before", "each_step", step)
        step.skip()
        self._hooks.call("after", "each_step", step)

    def exit(self):
        """
            Exits the runner
        """
        self._required_exit = True



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import time

import pytest

from radish.model import Model, Tag
from radish.exceptions import RadishError


def test_creating_simple_model():
    """
    Test creating a simple Model
    """
    # given & when
    model = Model(1, 'Model', 'I am a Model', 'foo.feature', 1, parent=None, tags=None)

    # then
    assert model.id == 1
    assert model.keyword == 'Model'
    assert model.sentence == 'I am a Model'
    assert model.path == 'foo.feature'
    assert model.line == 1
    assert model.parent is None
    assert model.tags == []


def test_creating_a_tag():
    """
    Test creating a Tag
    """
    # given & when
    tag = Tag('foo', arg='bar')

    # then
    assert tag.name == 'foo'
    assert tag.arg == 'bar'


def test_getting_tags_from_model():
    """
    Test getting all Tags from a Model
    """
    # given & when
    parent_model = Model(1, 'Model', 'I am a Model', 'foo.feature', 1, parent=None, tags=[Tag('some_tag')])
    model = Model(1, 'Model', 'I am a Model', 'foo.feature', 1, parent=parent_model, tags=[
        Tag('foo'), Tag('bar')])

    # when
    tags = model.all_tags

    # then
    assert len(tags) == 3
    assert tags[0].name == 'some_tag'
    assert tags[1].name == 'foo'
    assert tags[2].name == 'bar'


def test_getting_model_duration():
    """
    Test getting duration of a Model
    """
    # given & when
    model = Model(1, 'Model', 'I am a Model', 'foo.feature', 1, parent=None, tags=None)
    model.starttime = time.time()
    model.endtime = model.starttime + 10

    # when
    duration = model.duration

    # then
    assert duration == 10


def test_getting_model_duration_with_missing_time():
    """
    Test getting duration of a Model with missing start- or endtime
    """
    # given & when
    model = Model(1, 'Model', 'I am a Model', 'foo.feature', 1, parent=None, tags=None)

    # when - missing starttime
    model.starttime = None
    with pytest.raises(RadishError) as exc:
        model.duration
    # then
    assert str(exc.value) == "Cannot get duration of Model 'I am a Model' because either starttime or endtime is not set"

    # when - missing starttime
    model.starttime = time.time()
    with pytest.raises(RadishError) as exc:
        model.duration
    # then
    assert str(exc.value) == "Cannot get duration of Model 'I am a Model' because either starttime or endtime is not set"



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""


from radish import step


@step('Gegeben sei etwas')
def have_a_step(step):
    "Given I have a step"
    pass


@step('Wenn ich etwas mache')
def do_something(step):
    "When I do something"
    pass


@step('Dann erwarte ich etwas')
def expect_something(step):
    "Then I expect something"
    pass



# -*- coding: utf-8 -*-

from radish.stepregistry import step
from radish import given, when, then

@step("I have the following users")
def have_number(step):
    step.context.users = step.table

@when("I add them to the database")
def sum_numbres(step):
    for user in step.context.users:
        step.context.database.users.append({
            "forename": user['forename'],
            "surname": user['surname'],
            "hero": user['hero']})

@then("I expect {number:g} users in the database")
def expect_result(step, number):
    assert len(step.context.database.users) == number



# -*- coding: utf-8 -*-

"""
    This module provides a class to represent a Step
"""

import re

from .model import Model
from .exceptions import RadishError
from .terrain import world
from .stepregistry import StepRegistry
from .matcher import merge_step
from . import utils


class Step(Model):
    """
        Represents a step
    """
    class State(object):
        """
            Represents the step state

            FIXME: for the python3 version this should be an Enum
        """
        UNTESTED = "untested"
        SKIPPED = "skipped"
        PASSED = "passed"
        FAILED = "failed"
        PENDING = "pending"

    def __init__(self, id, sentence, path, line, parent, runable, context_class=None):
        super(Step, self).__init__(id, None, sentence, path, line, parent)
        self.context_class = context_class
        self.table_header = None
        self.table_data = []
        self.table = []
        self.raw_text = []
        self.definition_func = None
        self.argument_match = None
        self.state = Step.State.UNTESTED
        self.failure = None
        self.runable = runable
        self.as_precondition = None
        self.as_background = None

    @property
    def context(self):
        """
            Returns the scenario context belonging to this step
        """
        return self.parent.context

    @property
    def expanded_sentence(self):
        """
            Returns the expanded sentence of this step

                * Expand constants
        """
        sentence = self.sentence
        for name, value in self.parent.constants:
            sentence = sentence.replace("${{{0}}}".format(name), value)
        return sentence

    @property
    def context_sensitive_sentence(self):
        """
        Return the context class sensitive
        step sentence.
        """
        sentence = self.expanded_sentence
        if self.context_class:
            return utils.str_lreplace('and ', self.context_class.capitalize() + ' ', sentence, flags=re.IGNORECASE)

        return sentence

    @property
    def text(self):
        """
            Returns the additional text of this step as string
        """
        return "\n".join(self.raw_text)

    def _validate(self):
        """
            Checks if the step is valid to run or not
        """

        if not self.definition_func or not callable(self.definition_func):
            raise RadishError("The step '{0}' does not have a step definition".format(self.sentence))

    def run(self):
        """
            Runs the step.
        """
        if not self.runable:
            self.state = Step.State.UNTESTED
            return self.state

        self._validate()
        args, kwargs = self.argument_match.evaluate()

        try:
            if kwargs:
                self.definition_func(self, **kwargs)  # pylint: disable=not-callable
            else:
                self.definition_func(self, *args)  # pylint: disable=not-callable
        except Exception as e:  # pylint: disable=broad-except
            self.state = Step.State.FAILED
            self.failure = utils.Failure(e)
        else:
            if self.state is not Step.State.PENDING:
                self.state = Step.State.PASSED
        return self.state

    def debug(self):
        """
            Debugs the step
        """
        if not self.runable:
            self.state = Step.State.UNTESTED
            return self.state

        self._validate()
        args, kwargs = self.argument_match.evaluate()

        pdb = utils.get_debugger()

        try:
            pdb.runcall(self.definition_func, self, *args, **kwargs)
        except Exception as e:  # pylint: disable=broad-except
            self.state = Step.State.FAILED
            self.failure = utils.Failure(e)
        else:
            if self.state is not Step.State.PENDING:
                self.state = Step.State.PASSED
        return self.state

    def skip(self):
        """
            Skips the step
        """
        self.state = Step.State.SKIPPED

    def pending(self):
        """
            Mark the step as pending

            The end report will remind
            about all pending steps.
        """
        self.state = Step.State.PENDING

    def behave_like(self, sentence):
        """
            Make step behave like another one

            :param string sentence: the sentence of the step to behave like
        """
        # check if this step has already failed from a previous behave_like call
        if self.state is Step.State.FAILED:
            return

        # create step according to given sentence
        new_step = Step(None, sentence, self.path, self.line, self.parent, True)
        merge_step(new_step, StepRegistry().steps)

        # run or debug step
        if world.config.debug_steps:
            new_step.debug()
        else:
            new_step.run()

        # re-raise exception if the failed
        if new_step.state is Step.State.FAILED:
            new_step.failure.exception.args = ("Step '{0}' failed: '{1}'".format(sentence, new_step.failure.exception.message),)
            raise new_step.failure.exception



from radish import given, when, then


@given('I have the string {:QuotedString}')
def have_string(step, string):
    step.context.string = string


@when('I capitalize this string')
def capitalize_string(step):
    step.context.string = step.context.string.capitalize()


@then('I expect the string to be {:QuotedString}')
def expect_string(step, expected_string):
    assert step.context.string == expected_string, 'Expected "{0}" got "{1}"'.format(
        expected_string, step.context.string)



# -*- coding: utf-8 -*-

"""
    This radish extension module provide the functionality to write the end report
"""

# disable no-member lint error because of dynamic method from colorful
# pylint: disable=no-member

from datetime import timedelta

import colorful
import humanize

from radish.hookregistry import after
from radish.stepmodel import Step
from radish.utils import console_write as write, make_unique_obj_list, get_func_code
from radish.scenariooutline import ScenarioOutline
from radish.scenarioloop import ScenarioLoop
from radish.extensionregistry import extension
from radish.terrain import world
from radish.stepregistry import StepRegistry


@extension
class EndreportWriter(object):
    """
        Endreport writer radish extension
    """
    LOAD_IF = staticmethod(lambda config: not config.show)
    LOAD_PRIORITY = 50

    def __init__(self):
        after.all(self.console_write)

    def console_write(self, features, marker):
        """
            Writes the endreport for all features

            :param list features: all features
        """
        stats = {
            "features": {"amount": 0, "passed": 0, "failed": 0, "skipped": 0, "untested": 0, "pending": 0},
            "scenarios": {"amount": 0, "passed": 0, "failed": 0, "skipped": 0, "untested": 0, "pending": 0},
            "steps": {"amount": 0, "passed": 0, "failed": 0, "skipped": 0, "untested": 0, "pending": 0},
        }
        pending_steps = []
        duration = timedelta()
        for feature in features:
            if not feature.has_to_run(world.config.scenarios):
                continue
            stats["features"]["amount"] += 1
            stats["features"][feature.state] += 1

            if feature.state in [Step.State.PASSED, Step.State.FAILED]:
                duration += feature.duration

            for scenario in feature.all_scenarios:
                if not scenario.has_to_run(world.config.scenarios):
                    continue

                if isinstance(scenario, ScenarioOutline):  # skip ScenarioOutlines
                    continue
                if isinstance(scenario, ScenarioLoop):  # skip ScenarioLoop
                    continue

                stats["scenarios"]["amount"] += 1
                stats["scenarios"][scenario.state] += 1
                for step in scenario.steps:
                    stats["steps"]["amount"] += 1
                    stats["steps"][step.state] += 1

                    if step.state == Step.State.PENDING:
                        pending_steps.append(step)

        colored_closing_paren = colorful.bold_white(")")
        colored_comma = colorful.bold_white(", ")
        passed_word = colorful.bold_green("{0} passed")
        failed_word = colorful.bold_red("{0} failed")
        skipped_word = colorful.cyan("{0} skipped")
        pending_word = colorful.bold_yellow("{0} pending")

        output = colorful.bold_white("{0} features (".format(stats["features"]["amount"]))
        output += passed_word.format(stats["features"]["passed"])
        if stats["features"]["failed"]:
            output += colored_comma + failed_word.format(stats["features"]["failed"])
        if stats["features"]["skipped"]:
            output += colored_comma + skipped_word.format(stats["features"]["skipped"])
        if stats["features"]["pending"]:
            output += colored_comma + pending_word.format(stats["features"]["pending"])
        output += colored_closing_paren

        output += "\n"
        output += colorful.bold_white("{} scenarios (".format(stats["scenarios"]["amount"]))
        output += passed_word.format(stats["scenarios"]["passed"])
        if stats["scenarios"]["failed"]:
            output += colored_comma + failed_word.format(stats["scenarios"]["failed"])
        if stats["scenarios"]["skipped"]:
            output += colored_comma + skipped_word.format(stats["scenarios"]["skipped"])
        if stats["scenarios"]["pending"]:
            output += colored_comma + pending_word.format(stats["scenarios"]["pending"])
        output += colored_closing_paren

        output += "\n"
        output += colorful.bold_white("{} steps (".format(stats["steps"]["amount"]))
        output += passed_word.format(stats["steps"]["passed"])
        if stats["steps"]["failed"]:
            output += colored_comma + failed_word.format(stats["steps"]["failed"])
        if stats["steps"]["skipped"]:
            output += colored_comma + skipped_word.format(stats["steps"]["skipped"])
        if stats["steps"]["pending"]:
            output += colored_comma + pending_word.format(stats["steps"]["pending"])
        output += colored_closing_paren

        if pending_steps:
            sr = StepRegistry()
            pending_step_implementations = make_unique_obj_list(pending_steps, lambda x: x.definition_func)
            output += colorful.white("\nYou have {0} pending step implementation{1} affecting {2} step{3}:\n  {4}\n\nNote: this could be the reason for some failing subsequent steps".format(
                len(pending_step_implementations),
                "s" if len(pending_step_implementations) is not 1 else "",
                len(pending_steps),
                "s" if len(pending_steps) is not 1 else "",
                "\n  ".join(["-  '{0}' @ {1}".format(sr.get_pattern(s.definition_func), get_func_code(s.definition_func).co_filename) for s in pending_step_implementations])
            ))

        output += "\n"
        output += colorful.cyan("Run {0} finished within {1}".format(marker, humanize.naturaldelta(duration)))

        write(output)



# -*- coding: utf-8 -*-

from radish.stepregistry import step
from radish import when


@step("I have the number {number:g}")
def have_number(step, number):
    step.context.numbers.append(number)


@when("I sum them")
def sum_numbers(step):
    step.context.result = sum(step.context.numbers)



# -*- coding: utf-8 -*-

from radish.stepregistry import step
from radish import given, when, then

@step("I have the following quote")
def have_quote(step):
    step.context.quote = step.text

@when("I add it to the database")
def add_quote_to_db(step):
        step.context.database.quotes.append(step.context.quote)

@then("I expect {number:g} quote in the database")
def expect_amount_of_quotes(step, number):
    assert len(step.context.database.quotes) == number



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import os
import re
import tempfile

import pytest

import colorful

import radish.testing.matches as matches


@pytest.fixture(scope='module', autouse=True)
def disable_colors():
    """
    Fixture to disable colors
    """
    colorful.disable()
    yield
    colorful.use_8_ansi_colors()


def test_unreasonable_min_coverage(capsys):
    """
    Test unreasonable minimum test coverage
    """
    # given
    min_coverage = 101
    expected_returncode = 3

    # when
    actual_returncode = matches.test_step_matches_configs(None, [], min_coverage)
    _, err = capsys.readouterr()

    # then
    assert actual_returncode == expected_returncode
    assert err == 'You are a little cocky to think you can reach a minimum coverage of 101.00%\n'


def test_no_steps_found(mocker, capsys):
    """
    Test if basedir does not contain any Steps to test against
    """
    # given
    mocker.patch('radish.testing.matches.load_modules')
    expected_returncode = 4

    # when
    actual_returncode = matches.test_step_matches_configs(None, [])
    _, err = capsys.readouterr()

    # then
    assert actual_returncode == expected_returncode
    assert err == 'No step implementations found in [], thus doesn\'t make sense to continue'


def test_empty_matches_config(mocker, capsys):
    """
    Test empty matches config file
    """
    # given
    expected_returncode = 5

    fd, tmpfile = tempfile.mkstemp()
    os.close(fd)

    mocker.patch('radish.testing.matches.load_modules')
    steps_mock = mocker.patch('radish.testing.matches.StepRegistry.steps')
    steps_mock.return_value = [1, 2]

    # when
    actual_returncode = matches.test_step_matches_configs([tmpfile], [])
    out, _ = capsys.readouterr()

    # then
    assert actual_returncode == expected_returncode
    assert out == 'No sentences found in {0} to test against\n'.format(tmpfile)


@pytest.mark.parametrize('given_invalid_config, expected_error_msg', [
    ([{'sentence': None}],
     'You have to provide a sentence and the function name which should (not) be matched (should_match, should_not_match)'),
    ([{'should_match': None}],
     'You have to provide a sentence and the function name which should (not) be matched (should_match, should_not_match)'),
    ([{}],
     'You have to provide a sentence and the function name which should (not) be matched (should_match, should_not_match)'),
    ([{'foo': None}],
     'The config attributes foo are invalid. Use only sentence, should_match, should_not_match, with_arguments')
], ids=[
    'test config with missing should_match function attribute',
    'test config with missing sentence attribute',
    'test empty config',
    'test invalid config items'
])
def test_step_matches_invalid_config(given_invalid_config, expected_error_msg):
    """
    Test match config without a sentence and a should_match attribute
    """
    # given & when
    with pytest.raises(ValueError) as exc:
        matches.test_step_matches(given_invalid_config, [])

    # then
    assert str(exc.value) == expected_error_msg


def test_sentence_no_step_match(capsys):
    """
    Test if sentence does not match any Step Pattern
    """
    # given
    steps = {}
    config = [{
        'sentence': 'foo', 'should_match': 'bar'
    }]
    expected_returncode = (1, 0)

    # when
    actual_returncode = matches.test_step_matches(config, steps)
    out, _ = capsys.readouterr()

    # then
    assert 'Expected sentence didn\'t match any step implementation' in out
    assert actual_returncode == expected_returncode


def test_sentence_match_wrong_step(capsys):
    """
    Test if sentence matched wrong step
    """
    # given
    def foo(): pass

    steps = {'foo': foo}
    config = [{
        'sentence': 'foo', 'should_match': 'bar'
    }]
    expected_returncode = (1, 0)

    # when
    actual_returncode = matches.test_step_matches(config, steps)
    out, _ = capsys.readouterr()

    # then
    assert 'Expected sentence matched foo instead of bar' in out
    assert actual_returncode == expected_returncode


def test_sentence_argument_errors(capsys):
    """
    Test if sentence arguments do not match
    """
    # given
    def foo(step, foo, bar): pass

    steps = {re.compile(r'What (.*?) can (.*)'): foo}
    config = [{
        'sentence': 'What FOO can BAR', 'should_match': 'foo',
        'with_arguments': [
            {'foo': 'foooooooo'},
            {'bar': 'baaaaaaar'}
        ]
    }]
    expected_returncode = (1, 0)

    # when
    actual_returncode = matches.test_step_matches(config, steps)
    out, _ = capsys.readouterr()

    # then
    assert 'Expected argument "foo" with value "foooooooo" does not match value "FOO"' in out
    assert 'Expected argument "bar" with value "baaaaaaar" does not match value "BAR"' in out
    assert actual_returncode == expected_returncode


def test_sentence_not_match(capsys):
    """
    Test if sentence does not match if that's expected
    """
    # given
    def foo(step, foo, bar): pass

    steps = {re.compile(r'What (.*?) can (.*)'): foo}
    config = [{
        'sentence': 'foo', 'should_not_match': 'foo'
    }]
    expected_returncode = (0, 1)

    # when
    actual_returncode = matches.test_step_matches(config, steps)
    out, _ = capsys.readouterr()

    # then
    assert actual_returncode == expected_returncode


def test_sentence_not_match_anything(capsys):
    """
    Test if sentence does not match any steps
    """
    # given
    def foo(step, foo, bar): pass

    steps = {re.compile(r'What (.*?) can (.*)'): foo}
    config = [{
        'sentence': 'foo', 'should_not_match': ''
    }]
    expected_returncode = (0, 1)

    # when
    actual_returncode = matches.test_step_matches(config, steps)
    out, _ = capsys.readouterr()

    # then
    assert actual_returncode == expected_returncode


def test_sentence_not_match_specific_step(capsys):
    """
    Test if sentence does not match specific step
    """
    # given
    def foo(step): pass

    def bar(step): pass

    steps = {
        re.compile(r'foo'): foo,
        re.compile(r'bar'): bar,
    }
    config = [{
        'sentence': 'foo', 'should_not_match': 'bar'
    }]
    expected_returncode = (0, 1)

    # when
    actual_returncode = matches.test_step_matches(config, steps)
    out, _ = capsys.readouterr()

    # then
    assert actual_returncode == expected_returncode


def test_sentence_not_match_but_does(capsys):
    """
    Test if sentence matched step but shouldn't
    """
    # given
    def foo(): pass

    steps = {'foo': foo}
    config = [{
        'sentence': 'foo', 'should_not_match': 'foo'
    }]
    expected_returncode = (1, 0)

    # when
    actual_returncode = matches.test_step_matches(config, steps)
    out, _ = capsys.readouterr()

    # then
    assert 'Expected sentence did match foo but it shouldn\'t' in out
    assert actual_returncode == expected_returncode


@pytest.mark.parametrize('given_actual_arguments, expected_messages', [
    (
        {'FOO': None},
        ['Expected argument "foo" is not in matched arguments [\'FOO\']']
    ),
    (
        {'foo': 42},
        ['Expected argument "foo" is of type "int" instead "str"']
    ),
    (
        {'foo': 'foo'},
        ['Expected argument "foo" with value "fooo" does not match value "foo"']
    )
], ids=[
    'Missing argument',
    'Wrong type argument',
    'Wrong value argument'
])
def test_checking_step_arguments_errors(given_actual_arguments, expected_messages):
    """
    Test step argument checking errors
    """
    # given
    expected_arguments = {'foo': 'fooo'}

    # when
    messages = matches.check_step_arguments(expected_arguments, given_actual_arguments)

    # then
    assert messages == expected_messages


def test_checking_step_arguments():
    """
    Test sentence step arguments match
    """
    # given
    expected_arguments = {
        'foo': 'foo',
        'bar': 42
    }

    actual_arguments = {
        'foo': 'foo',
        'bar': 42
    }

    # when
    messages = matches.check_step_arguments(expected_arguments, actual_arguments)

    # then
    assert messages == []

def test_checking_step_arguments_directory():
    """
    Test sentence step argument directory match
    """
    #given
    expected_arguments = {
        'foobar': {'foo':'foo', 'bar':'bar'}
    }

    actual_arguments = {
        'foobar': {'foo':'foo', 'bar':'bar'}
    }

    #when
    messages = matches.check_step_arguments(expected_arguments, actual_arguments)

    #then
    assert messages == []





# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import pytest

from radish.scenariooutline import ScenarioOutline
from radish.examplescenario import ExampleScenario
from radish.background import Background
from radish.stepmodel import Step
from radish.exceptions import RadishError


def test_creating_simple_scenariooutline():
    """
    Test creating a simple ScenarioOutline
    """
    # given & when
    scenario = ScenarioOutline(1, 'Scenario Outline', 'Examples', 'I am a Scenario Outline', 'foo.feature', 1, parent=None,
                               tags=None, preconditions=None, background=None)

    # then
    assert scenario.id == 1
    assert scenario.keyword == 'Scenario Outline'
    assert scenario.example_keyword == 'Examples'
    assert scenario.sentence == 'I am a Scenario Outline'
    assert scenario.path == 'foo.feature'
    assert scenario.line == 1
    assert scenario.parent is None
    assert scenario.tags == []
    assert scenario.preconditions == []
    assert scenario.background is None


def test_building_scenariooutline_scenarios(mocker):
    """
    Test building Scenarios from a Scenario Outline Example
    """
    # given
    scenario_outline = ScenarioOutline(1, 'Scenario Outline', 'Examples', 'I am a Scenario Outline', 'foo.feature', 1, parent=None,
                                       tags=None, preconditions=None, background=None)
    # add steps
    scenario_outline.steps.extend([
        mocker.MagicMock(sentence='Given I have <foo>', path='foo.feature'),
        mocker.MagicMock(sentence='And I have <bar>', path='foo.feature'),
        mocker.MagicMock(sentence='When I add those', path='foo.feature')
    ])
    # add examples
    scenario_outline.examples_header = ['foo', 'bar']
    scenario_outline.examples = [
        # row 0
        ScenarioOutline.Example(['1', '2'], 'foo.feature', 1),
        # row 3
        ScenarioOutline.Example(['3', '4'], 'foo.feature', 2),
    ]

    # when - build the scenarios
    scenario_outline.build_scenarios()

    # then - expect 2 built Scenarios
    assert len(scenario_outline.scenarios) == 2
    # then - expect that Scenarios are of type ExampleScenario
    assert all(isinstance(x, ExampleScenario) for x in scenario_outline.scenarios)
    # then - expect correct Example Scenario sentences
    assert scenario_outline.scenarios[0].sentence == 'I am a Scenario Outline - row 0'
    assert scenario_outline.scenarios[1].sentence == 'I am a Scenario Outline - row 1'
    # then - expect correctly replaced Step sentences
    assert scenario_outline.scenarios[0].steps[0].sentence == 'Given I have 1'
    assert scenario_outline.scenarios[0].steps[1].sentence == 'And I have 2'
    assert scenario_outline.scenarios[0].steps[2].sentence == 'When I add those'
    assert scenario_outline.scenarios[1].steps[0].sentence == 'Given I have 3'
    assert scenario_outline.scenarios[1].steps[1].sentence == 'And I have 4'
    assert scenario_outline.scenarios[1].steps[2].sentence == 'When I add those'


def test_building_scenariooutline_scenarios_with_background(mocker):
    """
    Test building Scenarios from a Scenario Outline Example including a Background
    """
    # given
    background = Background('Background', 'I am a Background', 'foo.feature', 1, parent=None)
    # add some Steps
    background.steps.extend([
        Step(1, 'Foo', 'foo.feature', 2, background, False),
        Step(2, 'Foo', 'foo.feature', 3, background, False)
    ])
    scenario_outline = ScenarioOutline(1, 'Scenario Outline', 'Examples', 'I am a Scenario Outline', 'foo.feature', 1, parent=None,
                                       tags=None, preconditions=None, background=background)
    # add steps
    scenario_outline.steps.extend([
        mocker.MagicMock(sentence='Given I have <foo>', path='foo.feature'),
        mocker.MagicMock(sentence='And I have <bar>', path='foo.feature'),
        mocker.MagicMock(sentence='When I add those', path='foo.feature')
    ])
    # add examples
    scenario_outline.examples_header = ['foo', 'bar']
    scenario_outline.examples = [
        # row 0
        ScenarioOutline.Example(['1', '2'], 'foo.feature', 1),
        # row 3
        ScenarioOutline.Example(['3', '4'], 'foo.feature', 2),
    ]

    # when - build the scenarios
    scenario_outline.build_scenarios()

    # then - expect ExampleScenarios to have background copy assigned
    assert scenario_outline.scenarios[0].background.sentence == 'I am a Background'
    assert scenario_outline.scenarios[1].background.sentence == 'I am a Background'


def test_scenariooutline_example_colum_width():
    """
    Test calculation for maximum width of Example columns
    """
    # given
    scenario_outline = ScenarioOutline(1, 'Scenario Outline', 'Examples', 'I am a Scenario Outline', 'foo.feature', 1, parent=None,
                                       tags=None, preconditions=None, background=None)
    # add examples
    scenario_outline.examples_header = ['foo', 'bar']
    scenario_outline.examples = [
        # row 0
        ScenarioOutline.Example(['Spiderman', 'Batman'], 'foo.feature', 1),
        # row 3
        ScenarioOutline.Example(['Peter', 'Bruce Wayne'], 'foo.feature', 2),
    ]

    # when
    index_0_width = scenario_outline.get_column_width(0)
    # then
    assert index_0_width == len('Spiderman')

    # when
    index_1_width = scenario_outline.get_column_width(1)
    # then
    assert index_1_width == len('Bruce Wayne')


def test_scenariooutline_example_invalid_colum_width():
    """
    Test invalid column id exception when calculating column width for Scenario Outline Examples
    """
    # given
    scenario_outline = ScenarioOutline(1, 'Scenario Outline', 'Examples', 'I am a Scenario Outline', 'foo.feature', 1, parent=None,
                                       tags=None, preconditions=None, background=None)
    # add examples
    scenario_outline.examples_header = ['foo', 'bar']
    scenario_outline.examples = [
        # row 0
        ScenarioOutline.Example(['Spiderman', 'Batman'], 'foo.feature', 1),
        # row 3
        ScenarioOutline.Example(['Peter', 'Bruce Wayne'], 'foo.feature', 2),
    ]

    # when
    with pytest.raises(RadishError) as exc:
        scenario_outline.get_column_width(42)

    # then
    assert str(exc.value) == "Invalid colum_index to get column width for ScenarioOutline 'I am a Scenario Outline'"


def test_scenariooutline_afterparse_logic(mocker):
    """
    Test Scenario Outline after parse logic
    """
    # given
    scenario_outline = ScenarioOutline(1, 'Scenario Outline', 'Examples', 'I am a Scenario Outline', 'foo.feature', 1, parent=None,
                                       tags=None, preconditions=None, background=None)
    # add steps
    scenario_outline.steps.extend([
        mocker.MagicMock(sentence='Given I have <foo>', path='foo.feature'),
        mocker.MagicMock(sentence='And I have <bar>', path='foo.feature'),
        mocker.MagicMock(sentence='When I add those', path='foo.feature')
    ])
    # add examples
    scenario_outline.examples_header = ['foo', 'bar']
    scenario_outline.examples = [
        # row 0
        ScenarioOutline.Example(['1', '2'], 'foo.feature', 1),
        # row 3
        ScenarioOutline.Example(['3', '4'], 'foo.feature', 2),
    ]

    # when
    scenario_outline.after_parse()

    # then - expect 2 built Scenarios
    assert len(scenario_outline.scenarios) == 2
    assert scenario_outline.complete is True



# -*- coding: utf-8 -*-

"""
    This module provides an extension to write all features, scenarios and steps to the syslog.
"""

from radish.terrain import world
from radish.feature import Feature
from radish.hookregistry import before, after
from radish.extensionregistry import extension


@extension
class SyslogWriter(object):
    """
        Syslog Writer radish extension. This extension is only supported on
        systems where the Python standard library supports the system logger
        (syslog). For example, this extension works on UNIX and UNIX-like
        systems (Linux), but will not work on Windows.
    """

    OPTIONS = [("--syslog", "log all of your features, scenarios, and steps to the syslog")]
    LOAD_IF = staticmethod(lambda config: config.syslog)
    LOAD_PRIORITY = 40

    def __init__(self):
        # import syslog only if the extension got loaded
        # but not if the module got loaded.
        import syslog

        before.all(self.syslog_writer_before_all)
        before.each_feature(self.syslog_writer_before_each_feature)
        before.each_scenario(self.syslog_writer_before_each_scenario)
        before.each_step(self.syslog_writer_before_each_step)
        after.all(self.syslog_writer_after_all)
        after.each_feature(self.syslog_writer_after_each_feature)
        after.each_scenario(self.syslog_writer_after_each_scenario)
        after.each_step(self.syslog_writer_after_each_step)

    def get_scenario_feature(self, scenario):
        """
            Gets the scenarios feature
        """
        if not isinstance(scenario.parent, Feature):
            return scenario.parent.parent

        return scenario.parent

    def log(self, message):
        """
            Logs the given message to the syslog

            :param string message: the message to log
        """
        import syslog

        try:
            if isinstance(message, unicode):
                message = message.encode("utf8")
        except Exception:  # pylint: disable=broad-except
            pass
        finally:
            syslog.syslog(syslog.LOG_INFO, message)

    def syslog_writer_before_all(self, features, marker):  # pylint: disable=unused-argument
        """
            Opens the syslog
        """
        import syslog

        syslog.openlog("radish")
        self.log(u"begin run {0}".format(marker))

    def syslog_writer_after_all(self, features, marker):  # pylint: disable=unused-argument
        """
            Closes the syslog
        """
        import syslog

        self.log(u"end run {0}".format(marker))
        syslog.closelog()

    def syslog_writer_before_each_feature(self, feature):
        """
            Writes the feature to the syslog
        """
        self.log(u"begin feature {0}:{1} {2}".format(world.config.marker, feature.id, feature.sentence))

    def syslog_writer_after_each_feature(self, feature):
        """
            Writes the feature to the syslog
        """
        self.log(u"end feature {0}:{1} {2}".format(world.config.marker, feature.id, feature.sentence))

    def syslog_writer_before_each_scenario(self, scenario):
        """
            Writes the scenario to the syslog
        """
        self.log(u"begin scenario {0}:{1}.{2} {3}".format(world.config.marker, self.get_scenario_feature(scenario).id,
                                                          scenario.id, scenario.sentence))

    def syslog_writer_after_each_scenario(self, scenario):
        """
            Writes the scenario to the syslog
        """
        self.log(u"end scenario {0}:{1}.{2} {3}".format(world.config.marker, self.get_scenario_feature(scenario).id,
                                                        scenario.id, scenario.sentence))

    def syslog_writer_before_each_step(self, step):
        """
            Writes the step to the syslog
        """
        self.log(
            u"begin step {0}:{1}.{2}.{3} {4}".format(world.config.marker, self.get_scenario_feature(step.parent).id,
                                                     step.parent.id, step.id, step.sentence))

    def syslog_writer_after_each_step(self, step):
        """
            Writes the step to the syslog
        """
        self.log(u"{0} step {1}:{2}.{3}.{4} {5}".format(step.state, world.config.marker,
                                                        self.get_scenario_feature(step.parent).id, step.parent.id,
                                                        step.id, step.sentence))



# -*- coding: utf-8 -*-


"""
    This module provides an extension which starts a debugger when a step fails
"""

from radish.hookregistry import after
from radish.stepmodel import Step
from radish.extensionregistry import extension
import radish.utils as utils


@extension
class FailureDebugger(object):
    """
        Failure debugger radish extension
    """
    OPTIONS = [("--debug-after-failure", "start python debugger after failure")]
    LOAD_IF = staticmethod(lambda config: config.debug_after_failure)
    LOAD_PRIORITY = 20

    def __init__(self):
        after.each_step(self.failure_debugger)

    def failure_debugger(self, step):
        """
            Starts a python debugger if the step failed
        """
        if step.state is not Step.State.FAILED:
            return

        pdb = utils.get_debugger()
        pdb.set_trace()



# -*- coding: utf-8 -*-

"""
    This module provides a hook which generates a cucumber json result file at the end of the run.
"""

from getpass import getuser
from socket import gethostname
from datetime import timedelta
import re
import json
import logging

from radish.terrain import world
from radish.hookregistry import after
from radish.exceptions import RadishError
from radish.scenariooutline import ScenarioOutline
from radish.scenarioloop import ScenarioLoop
from radish.stepmodel import Step
from radish.extensionregistry import extension
import radish.utils as utils


@extension
class CucumberJSONWriter(object):
    """
        cucumber json Writer radish extension
    """
    OPTIONS = [("--cucumber-json=<ccjson>", "write cucumber json result file after run")]
    LOAD_IF = staticmethod(lambda config: config.cucumber_json)
    LOAD_PRIORITY = 60

    def __init__(self):
        after.all(self.generate_ccjson)

    def generate_ccjson(self, features, marker):
        """
            Generates the cucumber json
        """

        if not features:
            raise RadishError("No features given to generate cucumber json file")

        duration = timedelta()
        for feature in features:
            if feature.state in [Step.State.PASSED, Step.State.FAILED]:
                duration += feature.duration

        ccjson = []
        for feature in features:
            if not feature.has_to_run(world.config.scenarios):
                continue
            feature_description = "\n".join(feature.description)
            feature_json = {
                "uri": feature.path,
                "type": "feature",
                "keyword": feature.keyword,
                "id": str(feature.id),
                "name": feature.sentence,
                "line": feature.line,
                "description": feature_description,
                "tags": [],
                "elements": []
                }
            for i,j in enumerate(feature.tags):
                feature_json["tags"].append({"name": "@" + j.name, "line": feature.line - len(feature.tags) + i})
            for scenario in (s for s in feature.all_scenarios if not isinstance(s, (ScenarioOutline, ScenarioLoop))):
                if not scenario.has_to_run(world.config.scenarios):
                    continue
                scenario_json = {
                    "keyword": scenario.keyword,
                    "type": "scenario",
                    "id": str(scenario.id),
                    "name": scenario.sentence,
                    "line": scenario.line,
                    "description": "",
                    "steps": [],
                    "tags": []
                }
                start_line_no = scenario.line - len(scenario.tags)
                for i, tag in enumerate(scenario.tags):
                    scenario_json["tags"].append({"name": "@" + tag.name, "line": start_line_no + i})
                for step in scenario.all_steps:
                    duration = step.duration.total_seconds() * 1e9 if step.starttime and step.endtime else 0.0
                    step_json = {
                        "keyword": step.sentence.split()[0],
                        "name": step.sentence,
                        "line": step.line,
                        "result": {
                            "status": step.state,
                            "duration": duration
                        }
                    }
                    if step.state is Step.State.FAILED:
                        step_json["result"]["error_message"] = step.failure.reason
                    if step.state is Step.State.UNTESTED:
                        step_json["result"]["status"] = "skipped"
                    scenario_json["steps"].append(step_json)
                feature_json["elements"].append(scenario_json)
            ccjson.append(feature_json)

        with open(world.config.cucumber_json, "w+") as f:
            content = json.dumps(ccjson, indent=4, sort_keys=True)
            f.write(content)
